@@ -14,7 +14,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: ietf_constants.h,v 1.9 2004/12/09 18:17:30 mcr Exp $
+ * RCSID $Id: ietf_constants.h,v 1.16 2005/06/23 01:45:55 mcr Exp $
  */
 
 /* Group parameters from draft-ietf-ike-01.txt section 6 */
@@ -270,9 +270,9 @@ extern const char *const payload_name[];
 #define ISAKMP_NEXT_D          12	/* Delete */
 #define ISAKMP_NEXT_VID        13	/* Vendor ID */
 #define ISAKMP_NEXT_ATTR       14       /* Mode config Attribute */
-#define ISAKMP_NEXT_NATD_RFC   15       /* NAT-Traversal: NAT-D (rfc) */
-#define ISAKMP_NEXT_NATOA_RFC  16       /* NAT-Traversal: NAT-OA (rfc) */
-#define ISAKMP_NEXT_ROOF       17	/* roof on payload types */
+#define ISAKMP_NEXT_NATD_RFC   20       /* NAT-Traversal: NAT-D (rfc) */
+#define ISAKMP_NEXT_NATOA_RFC  21       /* NAT-Traversal: NAT-OA (rfc) */
+#define ISAKMP_NEXT_ROOF       22	/* roof on payload types */
 #define ISAKMP_NEXT_NATD_DRAFTS   130   /* NAT-Traversal: NAT-D (drafts) */
 #define ISAKMP_NEXT_NATOA_DRAFTS  131   /* NAT-Traversal: NAT-OA (drafts) */
 
@@ -352,6 +352,14 @@ extern enum_names exchange_names;
 /* added in draft-ietf-ipsec-ike-01.txt, near end of Appendix A */
 #define ISAKMP_XCHG_ACK_INFO   34	/* Oakley Acknowledged Informational */
 
+
+/* Private exchanges to pluto */
+#define ISAKMP_XCHG_ECHOREQUEST 30      /* Echo Request */
+#define ISAKMP_XCHG_ECHOREPLY   31      /* Echo Reply   */
+
+#define ISAKMP_XCHG_ECHOREQUEST_PRIVATE 244     /* Private Echo Request */
+#define ISAKMP_XCHG_ECHOREPLY_PRIVATE   245     /* Private Echo Reply   */
+
 /* Flag bits */
 
 extern const char *const flag_bit_names[];
@@ -529,6 +537,7 @@ extern enum_names auth_alg_names, extended_auth_alg_names;
 #define AUTH_ALGORITHM_HMAC_SHA2_512   7
 #define AUTH_ALGORITHM_HMAC_RIPEMD     8
 
+typedef u_int16_t ipsec_auth_t;
 
 /* Oakley Lifetime Type attribute
  * draft-ietf-ipsec-ike-01.txt appendix A
@@ -570,6 +579,9 @@ extern enum_names oakley_enc_names;
 #define OAKLEY_3DES_CBC         5
 #define OAKLEY_CAST_CBC         6
 #define OAKLEY_AES_CBC          7
+#define OAKLEY_SERPENT_CBC              65004
+#define OAKLEY_TWOFISH_CBC              65005
+#define OAKLEY_TWOFISH_CBC_SSH          65289
 
 #define OAKLEY_ENCRYPT_MAX      65535	/* pretty useless :) */
 
@@ -578,6 +590,7 @@ extern enum_names oakley_enc_names;
  * and from http://www.isi.edu/in-notes/iana/assignments/ipsec-registry
  */
 
+typedef u_int16_t oakley_hash_t;
 extern enum_names oakley_hash_names;
 
 #define OAKLEY_MD5      1
