@@ -12,19 +12,29 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: pluto_constants.h,v 1.22.2.1 2005/01/01 00:04:10 ken Exp $
+ * RCSID $Id: pluto_constants.h,v 1.33.2.2 2005/08/19 17:53:03 ken Exp $
  */
 
 /* Control and lock pathnames */
 
 #ifndef DEFAULT_CTLBASE
-# define DEFAULT_CTLBASE "/var/run/pluto"
+# define DEFAULT_CTLBASE "/var/run/pluto/pluto"
 #endif
 
 #define CTL_SUFFIX ".ctl"	/* for UNIX domain socket pathname */
 #define LOCK_SUFFIX ".pid"	/* for pluto's lock */
 #define INFO_SUFFIX ".info"     /* for UNIX domain socket for apps */
 
+enum kernel_interface {
+  NO_KERNEL = 1,
+  AUTO_PICK = 2,
+  USE_KLIPS = 3,
+  USE_NETKEY= 4
+};
+extern enum kernel_interface kern_interface;
+extern enum_names kern_interface_names;
+
+
 extern enum_names dpd_action_names;
 
 /* RFC 3706 Dead Peer Detection */
@@ -42,9 +52,7 @@ extern enum_names timer_event_names;
 enum event_type {
     EVENT_NULL,	/* non-event */
     EVENT_REINIT_SECRET,	/* Refresh cookie secret */
-#ifdef KLIPS
     EVENT_SHUNT_SCAN,	/* scan shunt eroutes known to kernel */
-#endif
     EVENT_SO_DISCARD,	/* discard unfinished state object */
     EVENT_RETRANSMIT,	/* Retransmit packet */
     EVENT_SA_REPLACE,	/* SA replacement event */
@@ -55,7 +63,8 @@ enum event_type {
     EVENT_DPD_TIMEOUT,	/* dead peer detection timeout */
 
     EVENT_LOG_DAILY,    /* reset certain log events/stats */
-    EVENT_CRYPTO_FAILED /* after some time, give up on crypto helper */
+    EVENT_CRYPTO_FAILED,/* after some time, give up on crypto helper */
+    EVENT_PENDING_PHASE2,  /* do not make pending phase2 wait forever */
 };
 
 #define EVENT_REINIT_SECRET_DELAY		3600 /* 1 hour */
@@ -72,6 +81,8 @@ enum pluto_crypto_requests {
   pcr_x509cert_fetch = 4,
   pcr_x509crl_fetch  = 5,
   pcr_build_nonce    = 6,
+  pcr_compute_dh_iv  = 7,  /* perform phase 1 calculation: DH + prf */
+  pcr_compute_dh     = 8,  /* perform phase 2 PFS DH */
 };
 
 extern enum_names pluto_cryptoop_names;
@@ -97,11 +108,17 @@ enum crypto_importance {
 
 typedef enum {
     STF_IGNORE,	/* don't respond */
+    STF_INLINE,        /* set to this on second time through complete_state_trans */
     STF_SUSPEND,    /* unfinished -- don't release resources */
     STF_OK,	/* success */
     STF_INTERNAL_ERROR,	/* discard everything, we failed */
-    STF_FAIL,	/* discard everything, something failed.  notification_t added. */
-    STF_FATAL,  /* just stop. we can't continue */
+    STF_TOOMUCHCRYPTO,  /* at this time, we can't do any more crypto,
+			 * so just ignore the message, and let them retransmit.
+			 */
+    STF_FATAL,          /* just stop. we can't continue. */
+    STF_FAIL,	        /* discard everything, something failed.  notification_t added.
+			 * values STF_FAIL + x are notifications.
+			 */
 } stf_status;
 extern enum_names stfstatus_name;
 
@@ -153,6 +170,8 @@ extern const char *const debug_bit_names[];
 #define IMPAIR_BUST_MR2	LELEM(IMPAIR0+3)	/* make MR2 really large */
 #define IMPAIR_SA_CREATION LELEM(IMPAIR0+4)     /* fail all SA creation */
 #define IMPAIR_DIE_ONINFO  LELEM(IMPAIR0+5)     /* cause state to be deleted upon receipt of information payload */
+#define IMPAIR_JACOB_TWO_TWO LELEM(IMPAIR0+6)   /* cause pluto to send all messages twice. */
+                                                /* cause pluto to send all messages twice. */
 
 #define DBG_NONE	0	/* no options on, including impairments */
 #define DBG_ALL		LRANGES(DBG_RAW, DBG_X509)  /* all logging options on EXCEPT DBG_PRIVATE */
@@ -240,6 +259,11 @@ enum state_kind {
 extern enum_names state_names;
 extern enum_names state_stories;
 
+enum phase1_role {
+  INITIATOR=1,
+  RESPONDER=2
+};
+
 
 #define STATE_IKE_FLOOR	STATE_MAIN_R0
 
@@ -401,7 +425,7 @@ extern const char *prettypolicy(lset_t policy);
 #define HAS_IPSEC_POLICY(p) (((p) & POLICY_IPSEC_MASK) != 0)
 
 /* Don't allow negotiation? */
-#define NEVER_NEGOTIATE(p)  (LDISJOINT((p), POLICY_PSK | POLICY_RSASIG | POLICY_AGGRESSIVE))
+#define NEVER_NEGOTIATE(p)  (LDISJOINT((p), POLICY_PSK | POLICY_RSASIG | POLICY_AGGRESSIVE) || (((p)&POLICY_SHUNT_MASK)!=POLICY_SHUNT_TRAP))
 
 
 /* Oakley transform attributes
