@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
  * License for more details.
  *
- * RCSID $Id: config-all.h,v 1.10 2004/08/22 20:12:46 mcr Exp $
+ * RCSID $Id: config-all.h,v 1.11 2005/01/26 00:52:59 mcr Exp $
  */
 #define	_CONFIG_ALL_H_	/* seen it, no need to see it again */
 
@@ -54,10 +54,6 @@
 #define CONFIG_KLIPS_ENC_3DES 1
 #endif
 
-#ifndef CONFIG_KLIPS_NAT_TRAVERSAL
-#define CONFIG_KLIPS_NAT_TRAVERSAL 1
-#endif
-
 #ifndef CONFIG_IPSEC_NAT_TRAVERSAL
 #define CONFIG_IPSEC_NAT_TRAVERSAL 1
 #endif
