@@ -11,7 +11,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: connections.h,v 1.98 2004/10/25 01:41:33 mcr Exp $
+ * RCSID $Id: connections.h,v 1.103 2005/06/14 22:38:06 mcr Exp $
  */
 
 /* There are two kinds of connections:
@@ -151,9 +151,10 @@ struct end {
     bool has_port_wildcard;
     bool has_id_wildcards;
     char *updown;
-    u_int16_t host_port;	/* host order */
-    u_int16_t port;		/* host order */
-    u_int8_t protocol;
+    u_int16_t host_port;	/* where the IKE port is */
+    bool      host_port_specific; /* if TRUE, then IKE ports are tested for*/
+    u_int16_t port;		/* port number, if per-port keying. */
+    u_int8_t protocol;          /* protocol number, if per-per keying. */
     cert_t cert;		/* end certificate */
     chunk_t ca;			/* CA distinguished name */
     struct ietfAttrList *groups;/* access control groups */
@@ -187,9 +188,9 @@ struct connection {
     unsigned long sa_keying_tries;
 
     /* RFC 3706 DPD */
-    time_t          dpd_delay;
-    time_t          dpd_timeout;
-    enum dpd_action dpd_action;
+    time_t          dpd_delay;              /* time between checks */
+    time_t          dpd_timeout;            /* time after which we are dead */
+    enum dpd_action dpd_action;             /* what to do when we die */
 
     bool               forceencaps;         /* always use NAT-T encap */
 
@@ -206,7 +207,7 @@ struct connection {
     policy_prio_t prio;
     bool instance_initiation_ok;	/* this is an instance of a policy that mandates initiate */
     enum connection_kind kind;
-    const struct iface *interface;	/* filled in iff oriented */
+    const struct iface_port *interface;	/* filled in iff oriented */
     bool initiated;
 
     so_serial_t	/* state object serial number */
@@ -259,6 +260,7 @@ struct whack_message;	/* forward declaration of tag whack_msg */
 extern void add_connection(const struct whack_message *wm);
 extern void initiate_connection(const char *name
 				, int whackfd
+				, lset_t moredebug
 				, enum crypto_importance importance);
 extern void initiate_opportunistic(const ip_address *our_client
     , const ip_address *peer_client, int transport_proto, bool held, int whackfd, err_t why);
@@ -287,8 +289,12 @@ extern void ISAKMP_SA_established(struct connection *c, so_serial_t serial);
 
 struct state;	/* forward declaration of tag (defined in state.h) */
 extern struct connection
-    *con_by_name(const char *nm, bool strict),
-    *find_host_connection(const ip_address *me, u_int16_t my_port
+*con_by_name(const char *nm, bool strict);
+
+#define find_host_connection(me, my_port, him, his_port) find_host_connection2(__FUNCTION__, me, my_port, him, his_port)
+extern struct connection 
+*find_host_connection2(const char *func
+		       , const ip_address *me, u_int16_t my_port
 	, const ip_address *him, u_int16_t his_port),
     *refine_host_connection(const struct state *st, const struct id *id
 	, bool initiator, bool aggrmode),
@@ -316,12 +322,7 @@ struct gw_info;	/* forward declaration of tag (defined in dnskey.h) */
 struct alg_info;	/* forward declaration of tag (defined in alg_info.h) */
 extern struct connection *rw_instantiate(struct connection *c
 					 , const ip_address *him
-#ifdef NAT_TRAVERSAL
-					 , u_int16_t his_port
-#endif
-#ifdef VIRTUAL_IP
 					 , const ip_subnet *his_net
-#endif					 
 					 , const struct id *his_id);
 
 extern struct connection *oppo_instantiate(struct connection *c
@@ -360,8 +361,6 @@ extern void release_pending_whacks(struct state *st, err_t story);
 extern void unpend(struct state *st);
 extern void update_pending(struct state *os, struct state *ns);
 extern void flush_pending_by_state(struct state *st);
-extern void show_pending_phase2(const struct host_pair *hp, const struct state *st);
-
 extern void connection_discard(struct connection *c);
 
 /* A template connection's eroute can be eclipsed by
@@ -385,6 +384,17 @@ update_host_pair(const char *why, struct connection *c,
        const ip_address *hisaddr, u_int16_t hisport);
 #endif /* NAT_TRAVERSAL */
 
+/* export to pending.c */
+void host_pair_enqueue_pending(const struct connection *c
+			       , struct pending *p
+			       , struct pending **pnext);
+struct pending **host_pair_first_pending(const struct connection *c);
+
+void connection_check_phase2(void);
+void init_connections(void);
+
+
+
 /*
  * Local Variables:
  * c-basic-offset:4
