@@ -11,7 +11,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: crypto.c,v 1.33 2004/11/25 18:18:58 mcr Exp $
+ * RCSID $Id: crypto.c,v 1.36 2005/07/05 22:04:13 mcr Exp $
  */
 
 #include <stdio.h>
@@ -54,40 +54,65 @@ static MP_INT
 MP_INT groupgenerator;	/* MODP group generator (2) */
 
 #ifdef IKE_ALG
+
+#ifdef USE_1DES
+static void do_des(u_int8_t *buf, size_t buf_len, u_int8_t *key, size_t key_size, u_int8_t *iv, bool enc);
+
+static struct encrypt_desc crypto_encrypter_des =
+{
+    common: {name: "oakley_des_cbc",
+             algo_type:         IKE_ALG_ENCRYPT,
+             algo_id:           OAKLEY_DES_CBC,
+             algo_next:         NULL, },
+    enc_ctxsize:        sizeof(des_key_schedule),
+    enc_blocksize:      DES_CBC_BLOCK_SIZE,
+    keydeflen:  DES_CBC_BLOCK_SIZE * BITS_PER_BYTE,
+    keyminlen:  DES_CBC_BLOCK_SIZE * BITS_PER_BYTE,
+    keymaxlen:  DES_CBC_BLOCK_SIZE * BITS_PER_BYTE,
+    do_crypt:   do_des,
+};
+#endif
+
+#ifdef USE_3DES
 static void do_3des(u_int8_t *buf, size_t buf_len, u_int8_t *key, size_t key_size, u_int8_t *iv, bool enc);
 static struct encrypt_desc crypto_encrypter_3des =
 { 	
-	common: {algo_type: 	IKE_ALG_ENCRYPT,
-		 algo_id:   	OAKLEY_3DES_CBC, 
-		 algo_next: 	NULL, },
-	enc_ctxsize: 	sizeof(des_key_schedule) * 3,
-	enc_blocksize: 	DES_CBC_BLOCK_SIZE, 
-	keydeflen: 	DES_CBC_BLOCK_SIZE * 3 * BITS_PER_BYTE,
-	keyminlen: 	DES_CBC_BLOCK_SIZE * 3 * BITS_PER_BYTE,
-	keymaxlen: 	DES_CBC_BLOCK_SIZE * 3 * BITS_PER_BYTE,
-	do_crypt: 	do_3des,
+    common: {name: "oakley_3des_cbc",
+	     algo_type: 	IKE_ALG_ENCRYPT,
+	     algo_id:   	OAKLEY_3DES_CBC, 
+	     algo_next: 	NULL, },
+    enc_ctxsize: 	sizeof(des_key_schedule) * 3,
+    enc_blocksize: 	DES_CBC_BLOCK_SIZE, 
+    keydeflen: 	DES_CBC_BLOCK_SIZE * 3 * BITS_PER_BYTE,
+    keyminlen: 	DES_CBC_BLOCK_SIZE * 3 * BITS_PER_BYTE,
+    keymaxlen: 	DES_CBC_BLOCK_SIZE * 3 * BITS_PER_BYTE,
+    do_crypt: 	do_3des,
 };
+#endif
+
 static struct hash_desc crypto_hasher_md5 =
 { 	
-	common: {algo_type: IKE_ALG_HASH,
-		 algo_id:   OAKLEY_MD5,
-		 algo_next: NULL, },
-	hash_ctx_size: sizeof(MD5_CTX),
-	hash_digest_len: MD5_DIGEST_SIZE,
-	hash_init: (void (*)(void *)) osMD5Init,
-	hash_update: (void (*)(void *, const u_int8_t *, size_t)) osMD5Update,
-	hash_final: (void (*)(u_char *, void *)) osMD5Final,
+    common: {name: "oakley_md5",
+	     algo_type: IKE_ALG_HASH,
+	     algo_id:   OAKLEY_MD5,
+	     algo_next: NULL, },
+    hash_ctx_size: sizeof(MD5_CTX),
+    hash_digest_len: MD5_DIGEST_SIZE,
+    hash_init: (void (*)(void *)) osMD5Init,
+    hash_update: (void (*)(void *, const u_int8_t *, size_t)) osMD5Update,
+    hash_final: (void (*)(u_char *, void *)) osMD5Final,
 };
 static struct hash_desc crypto_hasher_sha1 =
 { 	
-	common: {algo_type: IKE_ALG_HASH,
-		 algo_id:   OAKLEY_SHA,
-		 algo_next: NULL, },
-	hash_ctx_size: sizeof(SHA1_CTX),
-	hash_digest_len: SHA1_DIGEST_SIZE,
-	hash_init: (void (*)(void *)) SHA1Init,
-	hash_update: (void (*)(void *, const u_int8_t *, size_t)) SHA1Update,
-	hash_final: (void (*)(u_char *, void *)) SHA1Final,
+    common: {name: "oakley_sha",
+	     algo_type: IKE_ALG_HASH,
+	     algo_id:   OAKLEY_SHA,
+	     algo_next: NULL, },
+    hash_ctx_size: sizeof(SHA1_CTX),
+    hash_digest_len: SHA1_DIGEST_SIZE,
+    hash_init: (void (*)(void *)) SHA1Init,
+    hash_update: (void (*)(void *, const u_int8_t *, size_t)) SHA1Update,
+    hash_final: (void (*)(u_char *, void *)) SHA1Final,
 };
 #endif
 void
@@ -106,12 +131,57 @@ init_crypto(void)
     || mpz_init_set_str(&modp8192_modulus, MODP8192_MODULUS, 16) != 0)
 	exit_log("mpz_init_set_str() failed in init_crypto()");
 #ifdef IKE_ALG
-	{ 
-		extern int ike_alg_init(void);
+	{
+#ifdef USE_TWOFISH
+	    {
+		extern int ike_alg_twofish_init(void);
+		ike_alg_twofish_init();
+	    }
+#endif
+
+#ifdef USE_SERPENT
+	    {
+		extern int ike_alg_serpent_init(void);
+		ike_alg_serpent_init();
+	    }
+#endif
+
+#ifdef USE_AES
+	    {
+		extern int ike_alg_aes_init(void);
+		ike_alg_aes_init();
+	    }
+#endif
+
+#ifdef USE_3DES
+	    {
 		ike_alg_add((struct ike_alg *) &crypto_encrypter_3des);
-		ike_alg_add((struct ike_alg *) &crypto_hasher_md5);
-		ike_alg_add((struct ike_alg *) &crypto_hasher_sha1);
-		ike_alg_init();
+	    }
+#endif
+	    
+#ifdef USE_BLOWFISH
+	    {
+		extern int ike_alg_blowfish_init(void);
+		ike_alg_blowfish_init();
+	    }
+#endif
+
+#ifdef USE_1DES
+#error YOU HAVE TO EDIT THE SOURCE CODE AND REMOVE THIS LINE
+	    {
+		ike_alg_add((struct ike_alg *) &crypto_encrypter_des);
+	    }
+#endif
+
+#ifdef USE_SHA2
+	    {
+		extern int ike_alg_sha2_init(void);
+		ike_alg_sha2_init();
+	    }
+#endif
+	    
+	    ike_alg_add((struct ike_alg *) &crypto_hasher_sha1);
+	    ike_alg_add((struct ike_alg *) &crypto_hasher_md5);
 	}
 #endif
 }
@@ -155,23 +225,25 @@ lookup_group(u_int16_t group)
  * This must already be initialized.
  */
 
+#ifdef USE_1DES
 /* encrypt or decrypt part of an IKE message using DES
  * See RFC 2409 "IKE" Appendix B
  */
-static void __attribute__ ((unused))
-do_des(bool enc, void *buf, size_t buf_len, struct state *st)
+static void
+do_des(u_int8_t *buf, size_t buf_len, u_int8_t *key, size_t key_size, u_int8_t *iv, bool enc)
 {
     des_key_schedule ks;
 
-    (void) des_set_key((des_cblock *)st->st_enc_key.ptr, ks);
+    (void) des_set_key((des_cblock *)key, ks);
 
-    passert(st->st_new_iv_len >= DES_CBC_BLOCK_SIZE);
-    st->st_new_iv_len = DES_CBC_BLOCK_SIZE;	/* truncate */
+    passert(key_size >= DES_CBC_BLOCK_SIZE);
+    key_size = DES_CBC_BLOCK_SIZE;     /* truncate */
 
     des_ncbc_encrypt((des_cblock *)buf, (des_cblock *)buf, buf_len,
 	ks,
-	(des_cblock *)st->st_new_iv, enc);
+	(des_cblock *)iv, enc);
 }
+#endif
 
 /* encrypt or decrypt part of an IKE message using 3DES
  * See RFC 2409 "IKE" Appendix B
@@ -200,7 +272,7 @@ do_3des(u_int8_t *buf, size_t buf_len
  *
  *==========================================================
  */
-struct hash_desc *crypto_get_hasher(int alg)
+struct hash_desc *crypto_get_hasher(oakley_hash_t alg)
 {
 	return (struct hash_desc *) ike_alg_find(IKE_ALG_HASH, alg, 0);
 }
@@ -297,3 +369,12 @@ hmac_final(u_char *output, struct hmac_ctx *ctx)
     h->hash_update(&ctx->hash_ctx, output, h->hash_digest_len);
     h->hash_final(output, &ctx->hash_ctx);
 }
+
+
+/*
+ * Local Variables:
+ * c-basic-offset:4
+ * c-style: pluto
+ * End:
+ */
+
