@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: demux.c,v 1.187.2.1 2004/12/31 03:42:48 ken Exp $
+ * RCSID $Id: demux.c,v 1.210.2.7 2005/08/19 17:52:42 ken Exp $
  */
 
 /* Ordering Constraints on Payloads
@@ -158,6 +158,8 @@
 #include "nat_traversal.h"
 #endif
 #include "vendor.h"
+#include "dpd.h"
+#include "udpfromto.h"
 
 /* This file does basic header checking and demux of
  * incoming packets.
@@ -165,7 +167,6 @@
 
 /* forward declarations */
 static bool read_packet(struct msg_digest *md);
-static void process_packet(struct msg_digest **mdp);
 
 /* Reply messages are built in this buffer.
  * Only one state transition function can be using it at a time
@@ -246,6 +247,12 @@ static state_transition_fn	/* forward declaration */
 static const struct state_microcode
     *ike_microcode_index[STATE_IKE_ROOF - STATE_IKE_FLOOR];
 
+#define PHONY_STATE(X) \
+    { X, X \
+    , 0 \
+    , 0, P(VID) | P(CR), PT(NONE) \
+    , 0, NULL} 
+
 static const struct state_microcode state_microcode_table[] = {
 #define PT(n) ISAKMP_NEXT_##n
 #define P(n) LELEM(PT(n))
@@ -399,6 +406,7 @@ static const struct state_microcode state_microcode_table[] = {
 
     /* No state for aggr_outI1: -->HDR, SA, KE, Ni, IDii */
 
+#if defined(AGGRESSIVE)
     /* STATE_AGGR_R0:
      * SMF_PSK_AUTH: HDR, SA, KE, Ni, IDii
      *                -->  HDR, SA, KE, Nr, IDir, HASH_R
@@ -456,6 +464,19 @@ static const struct state_microcode state_microcode_table[] = {
     { STATE_AGGR_R2, STATE_UNDEFINED,
       SMF_ALL_AUTH,
       LEMPTY, LEMPTY, PT(NONE), EVENT_NULL, unexpected },
+#else
+    /*
+     * put in dummy states so that the state numbering does not
+     * change depending upon build options.
+     */
+    PHONY_STATE(STATE_AGGR_I1),
+    PHONY_STATE(STATE_AGGR_I1),
+    PHONY_STATE(STATE_AGGR_R1),
+    PHONY_STATE(STATE_AGGR_R1),
+    PHONY_STATE(STATE_AGGR_I2),
+    PHONY_STATE(STATE_AGGR_R2),
+#endif    
+    
 
 
     /***** Phase 2 Quick Mode *****/
@@ -671,7 +692,7 @@ init_demux(void)
 
 #if defined(IP_RECVERR) && defined(MSG_ERRQUEUE)
 static bool
-check_msg_errqueue(const struct iface *ifp, short interest)
+check_msg_errqueue(const struct iface_port *ifp, short interest)
 {
     struct pollfd pfd;
 
@@ -726,7 +747,7 @@ check_msg_errqueue(const struct iface *ifp, short interest)
 	if (packet_len == -1)
 	{
 	    log_errno((e, "recvmsg(,, MSG_ERRQUEUE) on %s failed in comm_handle"
-		, ifp->rname));
+		, ifp->ip_dev->id_rname));
 	    break;
 	}
 	else if (packet_len == sizeof(buffer))
@@ -775,12 +796,11 @@ check_msg_errqueue(const struct iface *ifp, short interest)
 	}
 
 	for (cm = CMSG_FIRSTHDR(&emh)
-	; cm != NULL
-	; cm = CMSG_NXTHDR(&emh,cm))
+		 ; cm != NULL
+		 ; cm = CMSG_NXTHDR(&emh,cm))
 	{
 	    if (cm->cmsg_level == SOL_IP
-	    && cm->cmsg_type == IP_RECVERR)
-	    {
+		&& cm->cmsg_type == IP_RECVERR)	{
 		/* ip(7) and recvmsg(2) specify:
 		 * ee_origin is SO_EE_ORIGIN_ICMP for ICMP
 		 *  or SO_EE_ORIGIN_LOCAL for locally generated errors.
@@ -872,7 +892,7 @@ check_msg_errqueue(const struct iface *ifp, short interest)
 		    else
 #endif
 		    openswan_log((sender != NULL) + "~"
-			"ERROR: asynchronous network error report on %s"
+			"ERROR: asynchronous network error report on %s (sport=%d)"
 			"%s"
 			", complainant %s"
 			": %s"
@@ -880,7 +900,8 @@ check_msg_errqueue(const struct iface *ifp, short interest)
 			/* ", pad %d, info %ld" */
 			/* ", data %ld" */
 			"]"
-			, ifp->rname
+			, ifp->ip_dev->id_rname
+				 , ifp->port
 			, fromstr
 			, offstr
 			, strerror(ee->ee_errno)
@@ -892,6 +913,9 @@ check_msg_errqueue(const struct iface *ifp, short interest)
 		    cur_state = old_state;
 		}
 	    }
+	    else if (cm->cmsg_level == SOL_IP
+		     && cm->cmsg_type == IP_PKTINFO) {
+	    }
 	    else
 	    {
 		/* .cmsg_len is a kernel_size_t(!), but the value
@@ -908,22 +932,15 @@ check_msg_errqueue(const struct iface *ifp, short interest)
 #endif /* defined(IP_RECVERR) && defined(MSG_ERRQUEUE) */
 
 bool
-#ifdef NAT_TRAVERSAL
-_send_packet(struct state *st, const char *where, bool verbose)
-#else
-send_packet(struct state *st, const char *where)
-#endif
+send_packet(struct state *st, const char *where, bool verbose)
 {
-    struct connection *c = st->st_connection;
-    int port_buf;
     bool err;
-
-#ifdef NAT_TRAVERSAL
     u_int8_t ike_pkt[MAX_OUTPUT_UDP_SIZE];
     u_int8_t *ptr;
     unsigned long len;
+    ssize_t wlen;
 
-    if ((c->interface->ike_float == TRUE) && (st->st_tpacket.len != 1)) {
+    if ((st->st_interface->ike_float == TRUE) && (st->st_tpacket.len != 1)) {
 	if ((unsigned long) st->st_tpacket.len >
 	    (MAX_OUTPUT_UDP_SIZE-sizeof(u_int32_t))) {
 	    DBG_log("send_packet(): really too big");
@@ -933,65 +950,81 @@ send_packet(struct state *st, const char *where)
 	/** Add Non-ESP marker **/
 	memset(ike_pkt, 0, sizeof(u_int32_t));
 	memcpy(ike_pkt + sizeof(u_int32_t), st->st_tpacket.ptr,
-	    (unsigned long)st->st_tpacket.len);
+	       (unsigned long)st->st_tpacket.len);
 	len = (unsigned long) st->st_tpacket.len + sizeof(u_int32_t);
     }
     else {
 	ptr = st->st_tpacket.ptr;
 	len = (unsigned long) st->st_tpacket.len;
     }
-#endif
 
-    DBG(DBG_RAW,
-	{
-	    DBG_log("sending %lu bytes for %s through %s to %s:%u:"
-		, (unsigned long) st->st_tpacket.len
-		, where
-		, c->interface->rname
- 		, ip_str(&c->spd.that.host_addr)
-		, (unsigned)c->spd.that.host_port);
-	    DBG_dump_chunk(NULL, st->st_tpacket);
-	});
+    DBG(DBG_CONTROL|DBG_RAW
+	, DBG_log("sending %lu bytes for %s through %s:%d to %s:%u:"
+		  , (unsigned long) st->st_tpacket.len
+		  , where
+		  , st->st_interface->ip_dev->id_rname
+		  , st->st_interface->port
+		  , ip_str(&st->st_remoteaddr)
+		  , st->st_remoteport));
+    DBG(DBG_RAW
+	, DBG_dump(NULL, ptr, len));
 
-    /* XXX: Not very clean.  We manipulate the port of the ip_address to
-     * have a port in the sockaddr*, but we retain the original port
-     * and restore it afterwards.
-     */
-
-    port_buf = portof(&c->spd.that.host_addr);
-    setportof(htons(c->spd.that.host_port), &c->spd.that.host_addr);
+    setportof(htons(st->st_remoteport), &st->st_remoteaddr);
 
 #if defined(IP_RECVERR) && defined(MSG_ERRQUEUE)
-    (void) check_msg_errqueue(c->interface, POLLOUT);
+    (void) check_msg_errqueue(st->st_interface, POLLOUT);
 #endif /* defined(IP_RECVERR) && defined(MSG_ERRQUEUE) */
 
-#ifdef NAT_TRAVERSAL
-    err = sendto(c->interface->fd
-    	, ptr, len, 0
-    	, sockaddrof(&c->spd.that.host_addr)
-    	, sockaddrlenof(&c->spd.that.host_addr)) != (ssize_t)len;
+#if 0
+    wlen = sendfromto(st->st_interface->fd
+		      , ptr
+		      , len, 0
+		      , sockaddrof(&st->st_remoteaddr)
+		      , sockaddrlenof(&st->st_remoteaddr)
+		      , sockaddrof(&st->st_localaddr)
+		      , sockaddrlenof(&st->st_localaddr));
 #else
-    err = sendto(c->interface->fd
-    	, st->st_tpacket.ptr, st->st_tpacket.len, 0
-    	, sockaddrof(&c->spd.that.host_addr)
-    	, sockaddrlenof(&c->spd.that.host_addr)) != (ssize_t)st->st_tpacket.len;
+    wlen = sendto(st->st_interface->fd
+		  , ptr
+		  , len, 0
+		  , sockaddrof(&st->st_remoteaddr)
+		  , sockaddrlenof(&st->st_remoteaddr));
+
+#ifdef DEBUG
+    if(DBGP(IMPAIR_JACOB_TWO_TWO)) {
+	/* sleep for half a second, and second another packet */
+	usleep(500000);
+
+	DBG_log("JACOB 2-2: resending %lu bytes for %s through %s:%d to %s:%u:"
+		, (unsigned long) st->st_tpacket.len
+		, where
+		, st->st_interface->ip_dev->id_rname
+		, st->st_interface->port
+		, ip_str(&st->st_remoteaddr)
+		, st->st_remoteport);
 #endif
 
-    /* restore port */
-    setportof(port_buf, &c->spd.that.host_addr);
+	wlen = sendto(st->st_interface->fd
+		      , ptr
+		      , len, 0
+		      , sockaddrof(&st->st_remoteaddr)
+		      , sockaddrlenof(&st->st_remoteaddr));
+    }
+
+	
+#endif
+    err = (wlen != (ssize_t)len);
 
     if (err)
     {
-#ifdef NAT_TRAVERSAL
         /* do not log NAT-T Keep Alive packets */
         if (!verbose)
 	    return FALSE; 
-#endif
 	log_errno((e, "sendto on %s to %s:%u failed in %s"
-	    , c->interface->rname
-	    , ip_str(&c->spd.that.host_addr)
-	    , (unsigned)c->spd.that.host_port
-	    , where));
+		   , st->st_interface->ip_dev->id_rname
+		   , ip_str(&st->st_remoteaddr)
+		   , st->st_remoteport
+		   , where));
 	return FALSE;
     }
     else
@@ -1027,6 +1060,11 @@ informational(struct msg_digest *md)
 	 * once we are at least in R3/I4. 
 	 * and that the handler is expected to treat them suspiciously.
 	 */
+	DBG(DBG_CONTROL, DBG_log("processing informational %s (%d)"
+				 , enum_name(&ipsec_notification_names
+					     ,n->isan_type)
+				 , n->isan_type));
+				 
         switch (n->isan_type)
         {
         case R_U_THERE:
@@ -1055,6 +1093,13 @@ informational(struct msg_digest *md)
 	    return STF_IGNORE;
 
         default:
+#ifdef DEBUG
+	    if(st!=NULL
+	       && st->st_connection->extra_debugging & IMPAIR_DIE_ONINFO) {
+		loglog(RC_LOG_SERIOUS, "received and failed on unknown informational message");
+		return STF_FATAL;
+	    }
+#endif	    
             if (pbs_left(n_pbs) >= sizeof(disp_buf)-1)
                 disp_len = sizeof(disp_buf)-1;
             else
@@ -1078,12 +1123,14 @@ static struct msg_digest *md_pool = NULL;
 void
 free_md_pool(void)
 {
+
     for (;;)
     {
 	struct msg_digest *md = md_pool;
 
 	if (md == NULL)
 	    break;
+	passert(md_pool != md->next);
 	md_pool = md->next;
 	pfree(md);
     }
@@ -1128,6 +1175,9 @@ release_md(struct msg_digest *md)
     passert(looking_for_state == NULL || md->st != looking_for_state);
     freeanychunk(md->raw_packet);
     pfreeany(md->packet_pbs.start);
+
+    /* make sure we are not creating a loop */
+    passert(md != md_pool);
     md->packet_pbs.start = NULL;
     md->next = md_pool;
     md_pool = md;
@@ -1147,7 +1197,7 @@ release_md(struct msg_digest *md)
  * enormous input packet buffer, an auto.
  */
 void
-comm_handle(const struct iface *ifp)
+comm_handle(const struct iface_port *ifp)
 {
     static struct msg_digest *md;
 
@@ -1188,7 +1238,7 @@ comm_handle(const struct iface *ifp)
 static bool
 read_packet(struct msg_digest *md)
 {
-    const struct iface *ifp = md->iface;
+    const struct iface_port *ifp = md->iface;
     int packet_len;
     /* ??? this buffer seems *way* too big */
     u_int8_t bigbuffer[MAX_INPUT_UDP_SIZE];
@@ -1200,15 +1250,20 @@ read_packet(struct msg_digest *md)
 	struct sockaddr sa;
 	struct sockaddr_in sa_in4;
 	struct sockaddr_in6 sa_in6;
-    } from;
+    } from,to;
     int from_len = sizeof(from);
+    int to_len   = sizeof(to);
     err_t from_ugh = NULL;
     static const char undisclosed[] = "unknown source";
 
-    happy(anyaddr(addrtypeof(&ifp->addr), &md->sender));
+    happy(anyaddr(addrtypeof(&ifp->ip_addr), &md->sender));
     zero(&from.sa);
-    packet_len = recvfrom(ifp->fd, bigbuffer, sizeof(bigbuffer), 0
-	, &from.sa, &from_len);
+    packet_len = recvfromto(ifp->fd, bigbuffer
+			    , sizeof(bigbuffer), /*flags*/0
+			    , &from.sa, &from_len
+			    , &to.sa, &to_len);
+
+    /* we do not do anything with *to* addresses yet... we will */
 
     /* First: digest the from address.
      * We presume that nothing here disturbs errno.
@@ -1243,12 +1298,16 @@ read_packet(struct msg_digest *md)
 	    {
 	    case AF_INET:
 		from_ugh = initaddr((void *) &from.sa_in4.sin_addr
-		    , sizeof(from.sa_in4.sin_addr), AF_INET, &md->sender);
+				    , sizeof(from.sa_in4.sin_addr)
+				    , AF_INET, &md->sender);
+		setportof(from.sa_in4.sin_port, &md->sender);
 		md->sender_port = ntohs(from.sa_in4.sin_port);
 		break;
 	    case AF_INET6:
 		from_ugh = initaddr((void *) &from.sa_in6.sin6_addr
-		    , sizeof(from.sa_in6.sin6_addr), AF_INET6, &md->sender);
+				    , sizeof(from.sa_in6.sin6_addr)
+				    , AF_INET6, &md->sender);
+		setportof(from.sa_in6.sin6_port, &md->sender);
 		md->sender_port = ntohs(from.sa_in6.sin6_port);
 		break;
 	    }
@@ -1270,12 +1329,12 @@ read_packet(struct msg_digest *md)
 	else if (from_ugh != NULL)
 	{
 	    log_errno((e, "recvfrom on %s failed; Pluto cannot decode source sockaddr in rejection: %s"
-		, ifp->rname, from_ugh));
+		, ifp->ip_dev->id_rname, from_ugh));
 	}
 	else
 	{
 	    log_errno((e, "recvfrom on %s from %s:%u failed"
-		, ifp->rname
+		, ifp->ip_dev->id_rname
 		, ip_str(&md->sender), (unsigned)md->sender_port));
 	}
 
@@ -1284,7 +1343,7 @@ read_packet(struct msg_digest *md)
     else if (from_ugh != NULL)
     {
 	openswan_log("recvfrom on %s returned misformed source sockaddr: %s"
-	    , ifp->rname, from_ugh);
+	    , ifp->ip_dev->id_rname, from_ugh);
 	return FALSE;
     }
     cur_from = &md->sender;
@@ -1323,10 +1382,11 @@ read_packet(struct msg_digest *md)
     DBG(DBG_RAW | DBG_CRYPT | DBG_PARSING | DBG_CONTROL,
 	{
 	    DBG_log(BLANK_FORMAT);
-	    DBG_log("*received %d bytes from %s:%u on %s"
-		, (int) pbs_room(&md->packet_pbs)
-		, ip_str(cur_from), (unsigned) cur_from_port
-		, ifp->rname);
+	    DBG_log("*received %d bytes from %s:%u on %s (port=%d)"
+		    , (int) pbs_room(&md->packet_pbs)
+		    , ip_str(cur_from), (unsigned) cur_from_port
+		    , ifp->ip_dev->id_rname
+		    , ifp->port);
 	});
 
     DBG(DBG_RAW,
@@ -1356,7 +1416,7 @@ read_packet(struct msg_digest *md)
  * If all goes well, this routine eventually calls a state-specific
  * transition function.
  */
-static void
+void
 process_packet(struct msg_digest **mdp)
 {
     struct msg_digest *md = *mdp;
@@ -1399,6 +1459,11 @@ process_packet(struct msg_digest **mdp)
 	return;
     }
 
+    DBG(DBG_CONTROL
+	, DBG_log(" processing packet with exchange type=%s (%d)"
+		  , enum_name(&exchange_names, md->hdr.isa_xchg)
+		  , md->hdr.isa_xchg));
+
     switch (md->hdr.isa_xchg)
     {
 #ifdef NOTYET
@@ -1655,8 +1720,8 @@ process_packet(struct msg_digest **mdp)
 	    return;
 	}
 
-	st = find_state(md->hdr.isa_icookie, md->hdr.isa_rcookie
-			, &md->sender, md->hdr.isa_msgid);
+	st = find_info_state(md->hdr.isa_icookie, md->hdr.isa_rcookie
+			     , &md->sender, md->hdr.isa_msgid);
 
 	if (st == NULL)
 	{
@@ -1665,7 +1730,7 @@ process_packet(struct msg_digest **mdp)
 	     * ??? what if this is a duplicate of another message?
 	     */
 	    st = find_state(md->hdr.isa_icookie, md->hdr.isa_rcookie
-			    , &md->sender, 0);
+				 , &md->sender, 0);
 
 	    if (st == NULL)
 	    {
@@ -1762,6 +1827,16 @@ process_packet(struct msg_digest **mdp)
 	break;
 #endif
 
+    case ISAKMP_XCHG_ECHOREQUEST_PRIVATE:
+    case ISAKMP_XCHG_ECHOREQUEST:
+	receive_ike_echo_request(md);
+	return;
+	
+    case ISAKMP_XCHG_ECHOREPLY_PRIVATE:
+    case ISAKMP_XCHG_ECHOREPLY:
+	receive_ike_echo_reply(md);
+	return;
+
 #ifdef NOTYET
     case ISAKMP_XCHG_NGRP:
     case ISAKMP_XCHG_ACK_INFO:
@@ -1801,7 +1876,11 @@ process_packet(struct msg_digest **mdp)
 
     if (st != NULL)
     {
+#if defined(XAUTH)
       oakley_auth_t baseauth = xauth_calcbaseauth(st->st_oakley.auth);
+#else
+      oakley_auth_t baseauth = st->st_oakley.auth;
+#endif
       while (!LHAS(smc->flags, baseauth))
 	{
 	  smc++;
@@ -1816,11 +1895,21 @@ process_packet(struct msg_digest **mdp)
      */
     if (st != NULL && st->st_suspended_md != NULL)
     {
-	loglog(RC_LOG, "discarding packet received during DNS lookup in %s"
+	loglog(RC_LOG, "discarding packet received during asynchronous work (DNS or crypto) in %s"
 	    , enum_name(&state_names, st->st_state));
 	return;
     }
 
+    /*
+     * if this state is busy calculating in between state transitions,
+     * (there will be no suspended state), then we silently ignore the
+     * packet, as there is nothing we can do right now.
+     */
+    if(st!=NULL && st->st_calculating) {
+	openswan_log("message received while calculating. Ignored.");
+	return;
+    }
+
     /* Detect and handle duplicated packets.
      * This won't work for the initial packet of an exchange
      * because we won't have a state object to remember it.
@@ -1832,9 +1921,9 @@ process_packet(struct msg_digest **mdp)
      * they are idempotent!
      */
     if (st != NULL
-    && st->st_rpacket.ptr != NULL
-    && st->st_rpacket.len == pbs_room(&md->packet_pbs)
-    && memcmp(st->st_rpacket.ptr, md->packet_pbs.start, st->st_rpacket.len) == 0)
+	&& st->st_rpacket.ptr != NULL
+	&& st->st_rpacket.len == pbs_room(&md->packet_pbs)
+	&& memcmp(st->st_rpacket.ptr, md->packet_pbs.start, st->st_rpacket.len) == 0)
     {
 	if (smc->flags & SMF_RETRANSMIT_ON_DUPLICATE)
 	{
@@ -1844,7 +1933,7 @@ process_packet(struct msg_digest **mdp)
 		loglog(RC_RETRANSMISSION
 		    , "retransmitting in response to duplicate packet; already %s"
 		    , enum_name(&state_names, st->st_state));
-		send_packet(st, "retransmit in response to duplicate");
+		send_packet(st, "retransmit in response to duplicate", TRUE);
 	    }
 	    else
 	    {
@@ -1860,6 +1949,28 @@ process_packet(struct msg_digest **mdp)
 	return;
     }
 
+    /*
+     * look for encrypt packets. We can not handle them if we have not
+     * yet calculated the skeyids. We will just store the packet in
+     * the suspended state, since the calculation is likely underway.
+     *
+     * note that this differs from above, because skeyid is calculated
+     * in between states. (or will be, once DH is async)
+     *
+     */
+    if((md->hdr.isa_flags & ISAKMP_FLAG_ENCRYPTION)
+       && st!=NULL && !st->hidden_variables.st_skeyid_calculated )
+    {
+	DBG(DBG_CRYPT|DBG_CONTROL
+	    , DBG_log("received encrypted packet from %s:%u but exponentiation still in progress"
+		      , ip_str(&md->sender), (unsigned)md->sender_port));
+
+	if(st->st_suspended_md) { release_md(st->st_suspended_md); }
+	st->st_suspended_md = md;
+	mdp = NULL;
+	return;
+    }
+
     if (md->hdr.isa_flags & ISAKMP_FLAG_ENCRYPTION)
     {
 	DBG(DBG_CRYPT, DBG_log("received encrypted packet from %s:%u"
@@ -1915,25 +2026,30 @@ process_packet(struct msg_digest **mdp)
 
 	    /* grab a copy of raw packet (for duplicate packet detection) */
 	    clonetochunk(md->raw_packet, md->packet_pbs.start
-		, pbs_room(&md->packet_pbs), "raw packet");
+			 , pbs_room(&md->packet_pbs), "raw packet");
 
 	    /* Decrypt everything after header */
 	    if (!new_iv_set)
 	    {
-		/* use old IV */
-		passert(st->st_iv_len <= sizeof(st->st_new_iv));
-		st->st_new_iv_len = st->st_iv_len;
-		init_new_iv(st);
-	    }
+		if(st->st_iv_len == 0) {
+		    init_phase2_iv(st, &md->hdr.isa_msgid);
+		} else {
+		    /* use old IV */
+		    passert(st->st_iv_len <= sizeof(st->st_new_iv));
+		    st->st_new_iv_len = st->st_iv_len;
+		    init_new_iv(st);
+		}
+	    } 
+
 	    crypto_cbc_encrypt(e, FALSE, md->message_pbs.cur, 
 			    pbs_left(&md->message_pbs) , st);
 	}
 
 	DBG_cond_dump(DBG_CRYPT, "decrypted:\n", md->message_pbs.cur
-	    , md->message_pbs.roof - md->message_pbs.cur);
+		      , md->message_pbs.roof - md->message_pbs.cur);
 
 	DBG_cond_dump(DBG_CRYPT, "next IV:"
-	    , st->st_new_iv, st->st_new_iv_len);
+		      , st->st_new_iv, st->st_new_iv_len);
     }
     else
     {
@@ -1978,10 +2094,10 @@ process_packet(struct msg_digest **mdp)
 	    {
 		case ISAKMP_NEXT_NATD_RFC:
 		case ISAKMP_NEXT_NATOA_RFC:
-		    if ((!st) || (!(st->nat_traversal & NAT_T_WITH_RFC_VALUES))) {
+		    if ((!st) || (!(st->hidden_variables.st_nat_traversal & NAT_T_WITH_RFC_VALUES))) {
 			/*
-			 * don't accept NAT-D/NAT-OA reloc directly in message, unless
-			 * we're using NAT-T RFC
+			 * don't accept NAT-D/NAT-OA reloc directly in message,
+			 * unless we're using NAT-T RFC
 			 */
 			sd = NULL;
 		    }
@@ -2225,6 +2341,32 @@ process_packet(struct msg_digest **mdp)
     complete_state_transition(mdp, smc->processor(md));
 }
 
+
+static void update_retransmit_history(struct state *st, struct msg_digest *md)
+{
+	/*
+	 * replace previous receive packet with latest, to update
+	 * our notion of a retransmitted packet. This is important
+	 * to do, even for failing transitions, and suspended transitions
+	 * because the sender may well retransmit their request.
+	 */
+	pfreeany(st->st_rpacket.ptr);
+	
+	if (md->encrypted)
+	{
+		/* if encrypted, duplication already done */
+		st->st_rpacket = md->raw_packet;
+		md->raw_packet.ptr = NULL;
+	}
+	else
+	{
+		clonetochunk(st->st_rpacket
+			     , md->packet_pbs.start
+			     , pbs_room(&md->packet_pbs), "raw packet");
+	}
+}	
+
+
 /* complete job started by the state-specific state transition function */
 
 void
@@ -2238,7 +2380,7 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
     cur_state = st = md->st;	/* might have changed */
 
     /* If state has DPD support, import it */
-    if( st && md->dpd) {
+    if( st && md->dpd && st->hidden_variables.st_dpd != md->dpd) {
 	DBG(DBG_DPD, DBG_log("peer supports dpd"));
 	st->hidden_variables.st_dpd = md->dpd;
 
@@ -2254,12 +2396,29 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 	, DBG_log("complete state transition with %s"
 		  , enum_name(&stfstatus_name, result)));
 
+    /*
+     * we can only be in calculating state if state is ignore,
+     * or suspended.
+     */
+    passert(result == STF_IGNORE || result == STF_SUSPEND || st->st_calculating==FALSE);
+
     switch (result)
     {
 	case STF_IGNORE:
 	    break;
 
+        case STF_INLINE:         /* this is second time through complete
+				  * state transition, so the MD has already
+				  * been freed.
+0				  */
+	    *mdp = NULL;
+	    break;
+
 	case STF_SUSPEND:
+	    /* update the previous packet history */
+	    if(md->packet_pbs.start) {
+                 update_retransmit_history(st, md);
+            }
 	    /* the stf didn't complete its job: don't relase md */
 	    *mdp = NULL;
 	    break;
@@ -2277,22 +2436,8 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 	     */
 	    delete_event(st);
 
-	    /* replace previous receive packet with latest */
-
-	    pfreeany(st->st_rpacket.ptr);
-
-	    if (md->encrypted)
-	    {
-		/* if encrypted, duplication already done */
-		st->st_rpacket = md->raw_packet;
-		md->raw_packet.ptr = NULL;
-	    }
-	    else
-	    {
-		clonetochunk(st->st_rpacket
-		    , md->packet_pbs.start
-		    , pbs_room(&md->packet_pbs), "raw packet");
-	    }
+	    /* update the previous packet history */
+	    update_retransmit_history(st, md);
 
 	    /* free previous transmit packet */
 	    freeanychunk(st->st_tpacket);
@@ -2300,7 +2445,15 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 	    /* if requested, send the new reply packet */
 	    if (smc->flags & SMF_REPLY)
 	    {
-		DBG(DBG_CONTROL, DBG_log("sending reply packet"));
+		char buf[ADDRTOT_BUF];
+
+		DBG(DBG_CONTROL
+		    , DBG_log("sending reply packet to %s:%u (from port=%d)"
+			      , (addrtot(&st->st_remoteaddr
+					 , 0, buf, sizeof(buf)), buf)
+			      , st->st_remoteport
+			      , st->st_interface->port));
+
 		close_output_pbs(&md->reply);   /* good form, but actually a no-op */
 
 		clonetochunk(st->st_tpacket, md->reply.start
@@ -2317,7 +2470,7 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 		 * for testing purposes.  Suppress or duplicate the
 		 * send_packet call depending on st->st_state.
 		 */
-		send_packet(st, enum_name(&state_names, from_state));
+		send_packet(st, enum_name(&state_names, from_state), TRUE);
 	    }
 
 	    /* Schedule for whatever timeout is specified */
@@ -2448,6 +2601,7 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 		
 		sadetails[0]='\0';
 
+		/* document IPsec SA details for admin's pleasure */
 		if(IS_IPSEC_SA_ESTABLISHED(st->st_state))
 		{
 		    char *b = sadetails;
@@ -2466,11 +2620,14 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 			    natinfo="/NAT";
 			}
 			snprintf(b, sizeof(sadetails)-(b-sadetails)-1
-				 , "%sESP%s=>0x%08x <0x%08x"
+				 , "%sESP%s=>0x%08x <0x%08x xfrm=%s_%d-%s"
 				 , ini
 				 , natinfo
 				 , ntohl(st->st_esp.attrs.spi)
-				 , ntohl(st->st_esp.our_spi));
+				 , ntohl(st->st_esp.our_spi)
+				 , enum_show(&esp_transformid_names, st->st_esp.attrs.transid)+strlen("ESP_")
+				 , st->st_esp.attrs.key_len
+				 , enum_show(&auth_alg_names, st->st_esp.attrs.auth)+strlen("AUTH_ALGORITHM_"));
 			ini = " ";
 			fin = "}";
 		    }
@@ -2504,43 +2661,81 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 		    /* advance b to end of string */
 		    b = b + strlen(b);
 #ifdef NAT_TRAVERSAL		    
-		    if(st->nat_traversal)
 		    {
 			char oa[ADDRTOT_BUF];
-			addrtot(&st->nat_oa, 0, oa, sizeof(oa));
+
+			strcpy(oa, "none");
+			if(!isanyaddr(&st->hidden_variables.st_nat_oa)) {
+			  addrtot(&st->hidden_variables.st_nat_oa, 0
+				  , oa, sizeof(oa));
+			}
 			snprintf(b, sizeof(sadetails)-(b-sadetails)-1
 				 , "%sNATOA=%s"
 				 , ini, oa);
 			ini = " ";
 			fin = "}";
 		    }
+
+		    {
+			char oa[ADDRTOT_BUF+sizeof(":00000")];
+
+			strcpy(oa, "none");
+			if(!isanyaddr(&st->hidden_variables.st_natd)) {
+			    char oa2[ADDRTOT_BUF];
+			    addrtot(&st->hidden_variables.st_natd, 0
+				    , oa2, sizeof(oa2));
+			    snprintf(oa, sizeof(oa)
+				     , "%s:%d", oa2, st->st_remoteport);
+			}
+			snprintf(b, sizeof(sadetails)-(b-sadetails)-1
+				 , "%sNATD=%s"
+				 , ini, oa);
+			ini = " ";
+			fin = "}";
+		    }
 #endif
 
 		    /* advance b to end of string */
 		    b = b + strlen(b);
 		    
-		    if(st->hidden_variables.st_dpd_local)
-		    {
-			snprintf(b, sizeof(sadetails)-(b-sadetails)-1
-				 , "%sDPD"
-				 , ini);
-			ini = " ";
-			fin = "}";
-		    }
+		    snprintf(b, sizeof(sadetails)-(b-sadetails)-1
+			     , "%sDPD=%s"
+			     , ini
+			     , st->hidden_variables.st_dpd_local ?
+			     "enabled" : "none");
+
+		    ini = " ";
+		    fin = "}";
 
 		    strcat(b, fin);
+		} else if(IS_ISAKMP_SA_ESTABLISHED(st->st_state)
+			  && !st->hidden_variables.st_logged_p1algos) {
+		    /* document ISAKMP SA details for admin's pleasure */
+		    char *b = sadetails;
+
+		    passert(st->st_oakley.encrypter != NULL);
+		    passert(st->st_oakley.hasher != NULL);
+		    passert(st->st_oakley.group != NULL);
+
+		    snprintf(b, sizeof(sadetails)-(b-sadetails)-1
+			     , " {auth=%s cipher=%s_%d prf=%s group=modp%d}"
+			     , enum_show(&oakley_auth_names, st->st_oakley.auth)
+			     , st->st_oakley.encrypter->common.name
+			     , st->st_oakley.enckeylen
+			     , st->st_oakley.hasher->common.name
+			     , (int)st->st_oakley.group->bytes*8);
+		    st->hidden_variables.st_logged_p1algos = TRUE;
 		}
 
 		if (IS_ISAKMP_SA_ESTABLISHED(st->st_state)
-		|| IS_IPSEC_SA_ESTABLISHED(st->st_state))
+		    || IS_IPSEC_SA_ESTABLISHED(st->st_state))
 		{
 		    /* log our success */
-		    openswan_log("%s%s", story, sadetails);
 		    w = RC_SUCCESS;
 		}
 
-		/* tell whack our progress */
-		whack_log(w
+		/* tell whack and logs our progress */
+		loglog(w
 		    , "%s: %s%s"
 		    , enum_name(&state_names, st->st_state)
 		    , story, sadetails);
@@ -2552,10 +2747,8 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 	     */
 	    if(IS_ISAKMP_SA_ESTABLISHED(st->st_state)) {
 		if(st->st_connection->dpd_delay>0
-		   && st->st_connection->dpd_timeout>0
-		   && st->st_dpd_event == NULL) {
-		    event_schedule(EVENT_DPD
-				   , st->st_connection->dpd_delay, st);
+		   && st->st_connection->dpd_timeout>0) {
+		    (void)dpd_init(st);
 		}
 	    }
 	     
@@ -2680,6 +2873,9 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 	    break;
 
 	case STF_INTERNAL_ERROR:
+	    /* update the previous packet history */
+	    update_retransmit_history(st, md);
+
 	    whack_log(RC_INTERNALERR + md->note
 		, "%s: internal error"
 		, enum_name(&state_names, st->st_state));
@@ -2689,12 +2885,26 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 		    , enum_name(&state_names, from_state)));
 	    break;
 
+        case STF_TOOMUCHCRYPTO:
+	    /* well, this should never happen during a whack, since
+	     * a whack will always force crypto.
+	     */
+            st->st_suspended_md = NULL;
+
+	    openswan_log("message in state %s ignored due to cryptographic overload"
+			 , enum_name(&state_names, from_state));
+	    break;
+
         case STF_FATAL:
-	    whack_log(RC_NOTIFICATION + md->note
-		      , "%s: %s", enum_name(&state_names, st->st_state)
-		      , enum_name(&ipsec_notification_names, md->note));
+	    /* update the previous packet history */
+	    update_retransmit_history(st, md);
+
+	    whack_log(RC_FATAL
+		      , "encountered fatal error in state %s"
+		      , enum_name(&state_names, st->st_state));
 	    delete_event(st);
-	    release_whack(st);
+	    release_pending_whacks(st, "fatal error");
+	    delete_state(st);
 	    break;
 
 	default:	/* a shortcut to STF_FAIL, setting md->note */
@@ -2711,7 +2921,9 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 		, "%s: %s", enum_name(&state_names, st->st_state)
 		, enum_name(&ipsec_notification_names, md->note));
 
-	    SEND_NOTIFICATION(md->note);
+	    if(md->note > 0) {
+		SEND_NOTIFICATION(md->note);
+	    }
 
 	    DBG(DBG_CONTROL,
 		DBG_log("state transition function for %s failed: %s"
@@ -2722,6 +2934,9 @@ complete_state_transition(struct msg_digest **mdp, stf_status result)
 		delete_event(st);
 		release_whack(st);
             }
+	    if(st!=NULL && IS_QUICK(st->st_state)) {
+		delete_state(st);
+	    }
             break;
     }
 }
