@@ -5,6 +5,7 @@
 struct connection;
 
 struct ike_alg {
+  const char *name;
   u_int16_t algo_type;	
   u_int16_t algo_id;	
   struct ike_alg *algo_next;
@@ -29,6 +30,9 @@ struct hash_desc {
   void (*hash_final)(u_int8_t *out, void *ctx);
 };
 
+struct alg_info_ike; /* forward reference */
+struct alg_info_esp;
+
 struct db_context * ike_alg_db_new(struct alg_info_ike *ai, lset_t policy);
 void ike_alg_show_status(void);
 void ike_alg_show_connection(struct connection *c, const char *instance);
@@ -71,5 +75,6 @@ extern struct db_sa *oakley_alg_makedb(struct alg_info_ike *ai
 				       , struct db_sa *basic
 				       , int maxtrans);
 
-extern struct db_sa *kernel_alg_makedb(struct alg_info_esp *ai);
+extern struct db_sa *kernel_alg_makedb(struct alg_info_esp *ai
+				       , bool logit);
 #endif /* _IKE_ALG_H */
