@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: kernel.c,v 1.215.2.1 2004/12/21 21:01:36 mcr Exp $
+ * RCSID $Id: kernel.c,v 1.232 2005/07/13 01:54:14 mcr Exp $
  */
 
 #include <stddef.h>
@@ -23,6 +23,7 @@
 #include <wait.h>
 #include <unistd.h>
 #include <fcntl.h>
+#include <sys/utsname.h>
 #include <sys/queue.h>
 
 #include <sys/stat.h>
@@ -71,31 +72,15 @@
 #include "nat_traversal.h"
 #endif
 
-#ifdef NAT_TRAVERSAL
-#include "packet.h"  /* for pb_stream in nat_traversal.h */
-#include "nat_traversal.h"
-#endif
-
 bool can_do_IPcomp = TRUE;  /* can system actually perform IPCOMP? */
 
-/* How far can IPsec messages arrive out of order before the anti-replay
- * logic loses track and swats them?  64 is the best KLIPS can do.
- */
-#define REPLAY_WINDOW   64
-
 /* test if the routes required for two different connections agree
  * It is assumed that the destination subnets agree; we are only
  * testing that the interfaces and nexthops match.
  */
-#define routes_agree(c, d) ((c)->interface == (d)->interface \
+#define routes_agree(c, d) ((c)->interface->ip_dev == (d)->interface->ip_dev \
         && sameaddr(&(c)->spd.this.host_nexthop, &(d)->spd.this.host_nexthop))
 
-#ifndef KLIPS
-
-bool no_klips = TRUE;   /* don't actually use KLIPS */
-
-#else /* !KLIPS */
-
 /* bare (connectionless) shunt (eroute) table
  *
  * Bare shunts are those that don't "belong" to a connection.
@@ -172,8 +157,6 @@ static void set_text_said(char *text_said
                           , ipsec_spi_t spi
                           , int proto);
 
-bool no_klips = FALSE;  /* don't actually use KLIPS */
-
 static const struct pfkey_proto_info null_proto_info[2] = {
         {
                 proto: IPPROTO_ESP,
@@ -252,8 +235,6 @@ record_and_initiate_opportunistic(const ip_subnet *ours
     }
 }
 
-#endif /* KLIPS */
-
 static unsigned get_proto_reqid(unsigned base, int proto)
 {
     switch (proto)
@@ -364,50 +345,12 @@ get_my_cpi(struct spd_route *sr, bool tunnel)
     return htonl((ipsec_spi_t)latest_cpi);
 }
 
-/* invoke the updown script to do the routing and firewall commands required
- *
- * The user-specified updown script is run.  Parameters are fed to it in
- * the form of environment variables.  All such environment variables
- * have names starting with "PLUTO_".
- *
- * The operation to be performed is specified by PLUTO_VERB.  This
- * verb has a suffix "-host" if the client on this end is just the
- * host; otherwise the suffix is "-client".  If the address family
- * of the host is IPv6, an extra suffix of "-v6" is added.
- *
- * "prepare-host" and "prepare-client" are used to delete a route
- * that may exist (due to forces outside of Pluto).  It is used to
- * prepare for pluto creating a route.
- *
- * "route-host" and "route-client" are used to install a route.
- * Since routing is based only on destination, the PLUTO_MY_CLIENT_*
- * values are probably of no use (using them may signify a bug).
- *
- * "unroute-host" and "unroute-client" are used to delete a route.
- * Since routing is based only on destination, the PLUTO_MY_CLIENT_*
- * values are probably of no use (using them may signify a bug).
- *
- * "up-host" and "up-client" are run when an eroute is added (not replaced).
- * They are useful for adjusting a firewall: usually for adding a rule
- * to let processed packets flow between clients.  Note that only
- * one eroute may exist for a pair of client subnets but inbound
- * IPsec SAs may persist without an eroute.
- *
- * "down-host" and "down-client" are run when an eroute is deleted.
- * They are useful for adjusting a firewall.
- */
-
-#ifndef DEFAULT_UPDOWN
-# define DEFAULT_UPDOWN "ipsec _updown"
-#endif
-
 static bool
-do_command(struct connection *c, struct spd_route *sr, const char *verb)
+do_command(struct connection *c, struct spd_route *sr, const char *verb, struct state *st)
 {
-    char cmd[1536];     /* arbitrary limit on shell command length */
     const char *verb_suffix;
 
-    /* figure out which verb suffix applies */
+    /* figure out which verb suffix applies for logging purposes */
     {
         const char *hs, *cs;
 
@@ -429,232 +372,18 @@ do_command(struct connection *c, struct spd_route *sr, const char *verb)
             ? hs : cs;
     }
 
-    /* form the command string */
-    {
-        char
-            nexthop_str[sizeof("PLUTO_NEXT_HOP='' ")+ADDRTOT_BUF],
-            me_str[ADDRTOT_BUF],
-            myid_str[IDTOA_BUF],
-            srcip_str[ADDRTOT_BUF+sizeof("PLUTO_MY_SOURCEIP=")+4],
-            myclient_str[SUBNETTOT_BUF],
-            myclientnet_str[ADDRTOT_BUF],
-            myclientmask_str[ADDRTOT_BUF],
-            peer_str[ADDRTOT_BUF],
-            peerid_str[IDTOA_BUF],
-            peerclient_str[SUBNETTOT_BUF],
-            peerclientnet_str[ADDRTOT_BUF],
-            peerclientmask_str[ADDRTOT_BUF],
-            secure_myid_str[IDTOA_BUF] = "",
-            secure_peerid_str[IDTOA_BUF] = "",
-            secure_peerca_str[IDTOA_BUF] = "";
-	    
-        ip_address ta;
+    DBG(DBG_CONTROL, DBG_log("command executing %s%s"
+			     , verb, verb_suffix));
 
-	nexthop_str[0]='\0';
-	if(addrbytesptr(&sr->this.host_nexthop, NULL)
-	   && !isanyaddr(&sr->this.host_nexthop))
-	{
-	    char *n;
-	    strcpy(nexthop_str, "PLUTO_NEXT_HOP='");
-	    n = nexthop_str + strlen(nexthop_str);
-	    addrtot(&sr->this.host_nexthop, 0,
-		    n, sizeof(nexthop_str)-strlen(nexthop_str));
-	    strncat(nexthop_str, "' ", sizeof(nexthop_str));
-	}
-
-        addrtot(&sr->this.host_addr, 0, me_str, sizeof(me_str));
-        idtoa(&sr->this.id, myid_str, sizeof(myid_str));
-        escape_metachar(myid_str, secure_myid_str, sizeof(secure_myid_str));
-        subnettot(&sr->this.client, 0, myclient_str, sizeof(myclientnet_str));
-        networkof(&sr->this.client, &ta);
-        addrtot(&ta, 0, myclientnet_str, sizeof(myclientnet_str));
-        maskof(&sr->this.client, &ta);
-        addrtot(&ta, 0, myclientmask_str, sizeof(myclientmask_str));
-
-        addrtot(&sr->that.host_addr, 0, peer_str, sizeof(peer_str));
-        idtoa(&sr->that.id, peerid_str, sizeof(peerid_str));
-        escape_metachar(peerid_str, secure_peerid_str, sizeof(secure_peerid_str));
-        subnettot(&sr->that.client, 0, peerclient_str, sizeof(peerclientnet_str));
-        networkof(&sr->that.client, &ta);
-        addrtot(&ta, 0, peerclientnet_str, sizeof(peerclientnet_str));
-        maskof(&sr->that.client, &ta);
-        addrtot(&ta, 0, peerclientmask_str, sizeof(peerclientmask_str));
-
-        srcip_str[0]='\0';
-        if(addrbytesptr(&sr->this.host_srcip, NULL) != 0
-           && !isanyaddr(&sr->this.host_srcip))
-        {
-            char *p;
-            int   l;
-            strncat(srcip_str, "PLUTO_MY_SOURCEIP=", sizeof(srcip_str));
-            strncat(srcip_str, "'", sizeof(srcip_str));
-            l = strlen(srcip_str);
-            p = srcip_str + l;
-            
-            addrtot(&sr->this.host_srcip, 0, p, sizeof(srcip_str));
-            strncat(srcip_str, "'", sizeof(srcip_str));
-        }
-
-        {
-            struct pubkey_list *p;
-            char peerca_str[IDTOA_BUF];
-
-            for (p = pubkeys; p != NULL; p = p->next)
-                {
-                    struct pubkey *key = p->key;
-                    int pathlen;
-                    
-                    if (key->alg == PUBKEY_ALG_RSA && same_id(&sr->that.id, &key->id)
-                        && trusted_ca(key->issuer, sr->that.ca, &pathlen))
-                        {
-                            dntoa_or_null(peerca_str, IDTOA_BUF, key->issuer, "");
-                            escape_metachar(peerca_str, secure_peerca_str, sizeof(secure_peerca_str));
-                            break;
-                        }
-                }
-        }
-
-        if (-1 == snprintf(cmd, sizeof(cmd)
-			   , "2>&1 "   /* capture stderr along with stdout */
-			   "PLUTO_VERSION='1.1' "    /* change VERSION when interface spec changes */
-			   "PLUTO_VERB='%s%s' "
-			   "PLUTO_CONNECTION='%s' "
-			   "%s"      /* possible PLUTO_NEXT_HOP */
-			   "PLUTO_INTERFACE='%s' "
-			   "PLUTO_ME='%s' "
-			   "PLUTO_MY_ID='%s' "
-			   "PLUTO_MY_CLIENT='%s' "
-			   "PLUTO_MY_CLIENT_NET='%s' "
-			   "PLUTO_MY_CLIENT_MASK='%s' "
-			   "PLUTO_MY_PORT='%u' "
-			   "PLUTO_MY_PROTOCOL='%u' "
-			   "PLUTO_PEER='%s' "
-			   "PLUTO_PEER_ID='%s' "
-			   "PLUTO_PEER_CLIENT='%s' "
-			   "PLUTO_PEER_CLIENT_NET='%s' "
-			   "PLUTO_PEER_CLIENT_MASK='%s' "
-			   "PLUTO_PEER_PORT='%u' "
-			   "PLUTO_PEER_PROTOCOL='%u' "
-			   "PLUTO_PEER_CA='%s' "
-			   "PLUTO_CONN_POLICY='%s' "
-			   "%s "       /* PLUTO_MY_SRCIP */                    
-			   "%s"        /* actual script */
-			   , verb, verb_suffix
-			   , c->name
-			   , nexthop_str
-			   , c->interface->vname
-			   , me_str
-			   , secure_myid_str
-			   , myclient_str
-			   , myclientnet_str
-			   , myclientmask_str
-			   , sr->this.port
-			   , sr->this.protocol
-			   , peer_str
-			   , secure_peerid_str
-			   , peerclient_str
-			   , peerclientnet_str
-			   , peerclientmask_str
-			   , sr->that.port
-			   , sr->that.protocol
-			   , secure_peerca_str
-			   , prettypolicy(c->policy)
-			   , srcip_str
-			   , sr->this.updown == NULL? DEFAULT_UPDOWN : sr->this.updown))
-        {
-            loglog(RC_LOG_SERIOUS, "%s%s command too long!", verb, verb_suffix);
-            return FALSE;
-        }
+    if(kernel_ops->docommand != NULL) {
+	return (*kernel_ops->docommand)(c,sr, verb, st);
+    } else {
+	DBG(DBG_CONTROL, DBG_log("no do_command for method %s", kernel_ops->opname));
     }
-
-    DBG(DBG_CONTROL, DBG_log("executing %s%s: %s"
-        , verb, verb_suffix, cmd));
-
-#ifdef KLIPS
-    if (!no_klips)
-    {
-        /* invoke the script, catching stderr and stdout
-         * It may be of concern that some file descriptors will
-         * be inherited.  For the ones under our control, we
-         * have done fcntl(fd, F_SETFD, FD_CLOEXEC) to prevent this.
-         * Any used by library routines (perhaps the resolver or syslog)
-         * will remain.
-         */
-        FILE *f = popen(cmd, "r");
-
-        if (f == NULL)
-        {
-            loglog(RC_LOG_SERIOUS, "unable to popen %s%s command", verb, verb_suffix);
-            return FALSE;
-        }
-
-        /* log any output */
-        for (;;)
-        {
-            /* if response doesn't fit in this buffer, it will be folded */
-            char resp[256];
-
-            if (fgets(resp, sizeof(resp), f) == NULL)
-            {
-                if (ferror(f))
-                {
-                    log_errno((e, "fgets failed on output of %s%s command"
-                        , verb, verb_suffix));
-                    return FALSE;
-                }
-                else
-                {
-                    passert(feof(f));
-                    break;
-                }
-            }
-            else
-            {
-                char *e = resp + strlen(resp);
-
-                if (e > resp && e[-1] == '\n')
-                    e[-1] = '\0';       /* trim trailing '\n' */
-                openswan_log("%s%s output: %s", verb, verb_suffix, resp);
-            }
-        }
-
-        /* report on and react to return code */
-        {
-            int r = pclose(f);
-
-            if (r == -1)
-            {
-                log_errno((e, "pclose failed for %s%s command"
-                    , verb, verb_suffix));
-                return FALSE;
-            }
-            else if (WIFEXITED(r))
-            {
-                if (WEXITSTATUS(r) != 0)
-                {
-                    loglog(RC_LOG_SERIOUS, "%s%s command exited with status %d"
-                        , verb, verb_suffix, WEXITSTATUS(r));
-                    return FALSE;
-                }
-            }
-            else if (WIFSIGNALED(r))
-            {
-                loglog(RC_LOG_SERIOUS, "%s%s command exited with signal %d"
-                    , verb, verb_suffix, WTERMSIG(r));
-                return FALSE;
-            }
-            else
-            {
-                loglog(RC_LOG_SERIOUS, "%s%s command exited with unknown status %d"
-                    , verb, verb_suffix, r);
-                return FALSE;
-            }
-        }
-    }
-#endif /* KLIPS */
     return TRUE;
 }
 
+
 /* Check that we can route (and eroute).  Diagnose if we cannot. */
 
 enum routability {
@@ -703,7 +432,7 @@ could_route(struct connection *c)
     }
 
     /* if routing would affect IKE messages, reject */
-    if (!no_klips
+    if (kern_interface != NO_KERNEL
 #ifdef NAT_TRAVERSAL
     && c->spd.this.host_port != NAT_T_IKE_FLOAT_PORT
 #endif
@@ -869,7 +598,7 @@ unroute_connection(struct connection *c)
 
         /* only unroute if no other connection shares it */
         if (routed(cr) && route_owner(c, NULL, NULL, NULL) == NULL)
-            (void) do_command(c, sr, "unroute");
+            (void) do_command(c, sr, "unroute", NULL);
     }
 }
 
@@ -1273,6 +1002,10 @@ sag_eroute(struct state *st, struct spd_route *sr
     proto_info[i].proto = 0;
     tunnel = FALSE;
 
+    inner_proto = 0;
+    inner_satype= 0;
+    inner_spi = 0;
+
     if (st->st_ah.present)
     {
         inner_spi = st->st_ah.attrs.spi;
@@ -1565,6 +1298,8 @@ scan_proc_shunts(void)
         err_t context = ""
             , ugh = NULL;
 
+	ff = NULL;
+
         cp = fgets(buf, sizeof(buf), f);
         if (cp == NULL)
             break;
@@ -1793,8 +1528,8 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
     struct connection *c = st->st_connection;
     ip_subnet src, dst;
     ip_subnet src_client, dst_client;
-    ipsec_spi_t inner_spi;
-    unsigned int proto, satype;
+    ipsec_spi_t inner_spi = 0;
+    unsigned int proto = 0, satype = 0;
     bool replace;
 
     /* SPIs, saved for spigrouping or undoing, if necessary */
@@ -1986,15 +1721,31 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
         u_int16_t natt_sport = 0, natt_dport = 0;
         ip_address natt_oa;
 
-        if (st->nat_traversal & NAT_T_DETECTED) {
-            natt_type = (st->nat_traversal & NAT_T_WITH_PORT_FLOATING) ?
-                ESPINUDP_WITH_NON_ESP : ESPINUDP_WITH_NON_IKE;
-            natt_sport = inbound? c->spd.that.host_port : c->spd.this.host_port;
-            natt_dport = inbound? c->spd.this.host_port : c->spd.that.host_port;
-            natt_oa = st->nat_oa;
+        if (st->hidden_variables.st_nat_traversal & NAT_T_DETECTED) {
+	    if(st->hidden_variables.st_nat_traversal & NAT_T_WITH_PORT_FLOATING) {
+		natt_type = ESPINUDP_WITH_NON_ESP;
+	    } else {
+		natt_type = ESPINUDP_WITH_NON_IKE;
+	    }
+	       
+	    if(inbound) {
+		natt_sport = st->st_remoteport;
+		natt_dport = st->st_localport;
+	    } else {
+		natt_sport = st->st_localport;
+		natt_dport = st->st_remoteport;
+	    }
+		
+            natt_oa = st->hidden_variables.st_nat_oa;
         }
 #endif
 
+	DBG(DBG_CRYPT
+	    , DBG_log("looking for alg with transid: %d keylen: %d auth: %d\n"
+		      , st->st_esp.attrs.transid
+		      , st->st_esp.attrs.key_len
+		      , st->st_esp.attrs.auth));
+		    
         for (ei = esp_info; ; ei++)
         {
 	    
@@ -2015,14 +1766,19 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
                  * enum_name does the same job, without a static buffer,
                  * assuming the name will be found.
                  */
-                loglog(RC_LOG_SERIOUS, "ESP transform %s / auth %s not implemented yet"
+                loglog(RC_LOG_SERIOUS, "ESP transform %s(%d) / auth %s not implemented yet"
                     , enum_name(&esp_transformid_names, st->st_esp.attrs.transid)
+		       , st->st_esp.attrs.key_len
                     , enum_name(&auth_alg_names, st->st_esp.attrs.auth));
                 goto fail;
             }
 
+	    DBG(DBG_CRYPT
+		, DBG_log("checking transid: %d keylen: %d auth: %d\n"
+			  , ei->transid, ei->enckeylen, ei->auth));
+		    
             if (st->st_esp.attrs.transid == ei->transid
-		&& st->st_esp.attrs.key_len == ei->enckeylen*8  
+		&& (st->st_esp.attrs.key_len ==0 || st->st_esp.attrs.key_len == ei->enckeylen*8)
 		&& st->st_esp.attrs.auth == ei->auth)
                 break;
         }
@@ -2073,7 +1829,7 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
         said_next->dst_client = &dst_client;
         said_next->spi = esp_spi;
         said_next->satype = SADB_SATYPE_ESP;
-        said_next->replay_window = REPLAY_WINDOW;
+        said_next->replay_window = kernel_ops->replay_window;
         said_next->authalg = ei->authalg;
         said_next->authkeylen = ei->authkeylen;
         /* said_next->authkey = esp_dst_keymat + ei->enckeylen; */
@@ -2136,7 +1892,7 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
         said_next->dst_client = &dst_client;
         said_next->spi = ah_spi;
         said_next->satype = SADB_SATYPE_AH;
-        said_next->replay_window = REPLAY_WINDOW;
+        said_next->replay_window = kernel_ops->replay_window;
         said_next->authalg = authalg;
         said_next->authkeylen = st->st_ah.keymat_len;
         said_next->authkey = ah_dst_keymat;
@@ -2251,14 +2007,24 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
         /* could update said, but it will not be used */
     }
 
+#ifdef DEBUG
+    /* if the impaired is set, pretend this fails */
+    if(st->st_connection->extra_debugging & IMPAIR_SA_CREATION) {
+	DBG_log("Impair SA creation is set, pretending to fail");
+	goto fail;
+    }
+#endif
     return TRUE;
 
 fail:
     {
         /* undo the done SPIs */
-        while (said_next-- != said)
-            (void) del_spi(said_next->spi, said_next->proto
-                , &src.addr, said_next->dst);
+        while (said_next-- != said) {
+	    if(said_next->proto) {
+		(void) del_spi(said_next->spi, said_next->proto
+			       , &src.addr, said_next->dst);
+	    }
+	}
         return FALSE;
     }
 }
@@ -2364,12 +2130,19 @@ const struct kernel_ops *kernel_ops;
 
 #endif /* KLIPS */
 
+/* keep track of kernel version */
+char kversion[256];
+
 void
 init_kernel(void)
 {
-#ifdef KLIPS
+    struct utsname un;
 
-    if (no_klips)
+    /* get kernel version */
+    uname(&un);
+    strncpy(kversion, un.release, sizeof(kversion));
+
+    if (kern_interface == NO_KERNEL)
     {
         kernel_ops = &noklips_kernel_ops;
         return;
@@ -2377,9 +2150,8 @@ init_kernel(void)
 
     init_pfkey();
 
-    kernel_ops = &klips_kernel_ops;
-
-#if defined(linux) && defined(KERNEL26_SUPPORT)
+#if defined(KLIPS) && defined(KERNEL26_SUPPORT)
+    if(kern_interface == AUTO_PICK)
     {
         bool linux_ipsec = 0;
         struct stat buf;
@@ -2387,16 +2159,38 @@ init_kernel(void)
         linux_ipsec = (stat("/proc/net/pfkey", &buf) == 0);
         if (linux_ipsec)
             {
-                openswan_log("Using Linux 2.6 IPsec interface code");
-                kernel_ops = &linux_kernel_ops;
+		kern_interface = USE_NETKEY;
             }
         else
             {
-                openswan_log("Using KLIPS IPsec interface code");
+                kern_interface = USE_KLIPS;
             }
     }
 #endif
 
+    switch(kern_interface) {
+#if defined(KERNEL26_SUPPORT)
+    case USE_NETKEY:
+	openswan_log("Using Linux 2.6 IPsec interface code on %s"
+		     , kversion);
+	kernel_ops = &linux_kernel_ops;
+	break;
+#endif
+
+#if defined(KLIPS) 
+    case USE_KLIPS:
+	openswan_log("Using KLIPS IPsec interface code on %s"
+		     , kversion);
+	kernel_ops = &klips_kernel_ops;
+	break;
+#endif
+
+    default:
+	openswan_log("kernel interface '%s' not available"
+		     , enum_name(&kern_interface_names, kern_interface));
+	exit(5);
+    }
+
     if (kernel_ops->init)
     {
         kernel_ops->init();
@@ -2410,7 +2204,6 @@ init_kernel(void)
     {
         event_schedule(EVENT_SHUNT_SCAN, SHUNT_SCAN_INTERVAL, NULL);
     }
-#endif
 }
 
 /* Note: install_inbound_ipsec_sa is only used by the Responder.
@@ -2578,7 +2371,7 @@ route_and_eroute(struct connection *c USED_BY_KLIPS
          */
         firewall_notified = st == NULL  /* not a tunnel eroute */
             || sr->eroute_owner != SOS_NOBODY   /* already notified */
-            || do_command(c, sr, "up"); /* go ahead and notify */
+            || do_command(c, sr, "up", st); /* go ahead and notify */
     }
 
     /* install the route */
@@ -2593,8 +2386,8 @@ route_and_eroute(struct connection *c USED_BY_KLIPS
     else if (ro == NULL)
     {
         /* a new route: no deletion required, but preparation is */
-        (void) do_command(c, sr, "prepare");    /* just in case; ignore failure */
-        route_installed = do_command(c, sr, "route");
+        (void) do_command(c, sr, "prepare", st);    /* just in case; ignore failure */
+        route_installed = do_command(c, sr, "route", st);
     }
     else if (routed(sr->routing)
     || routes_agree(ro, c))
@@ -2614,13 +2407,13 @@ route_and_eroute(struct connection *c USED_BY_KLIPS
          */
         if (sameaddr(&sr->this.host_nexthop, &esr->this.host_nexthop))
         {
-            (void) do_command(ro, sr, "unroute");
-            route_installed = do_command(c, sr, "route");
+            (void) do_command(ro, sr, "unroute", st);
+            route_installed = do_command(c, sr, "route", st);
         }
         else
         {
-            route_installed = do_command(c, sr, "route");
-            (void) do_command(ro, sr, "unroute");
+            route_installed = do_command(c, sr, "route", st);
+            (void) do_command(ro, sr, "unroute", st);
         }
 
         /* record unrouting */
@@ -2691,7 +2484,7 @@ route_and_eroute(struct connection *c USED_BY_KLIPS
     {
         /* Failure!  Unwind our work. */
         if (firewall_notified && sr->eroute_owner == SOS_NOBODY)
-            (void) do_command(c, sr, "down");
+            (void) do_command(c, sr, "down", st);
 
         if (eroute_installed)
         {
@@ -2867,7 +2660,7 @@ delete_ipsec_sa(struct state *st USED_BY_KLIPS, bool inbound_only USED_BY_KLIPS)
                 sr->routing = (c->policy & POLICY_FAIL_MASK) == POLICY_FAIL_NONE
                     ? RT_ROUTED_PROSPECTIVE : RT_ROUTED_FAILURE;
 
-                (void) do_command(c, sr, "down");
+                (void) do_command(c, sr, "down", st);
                 if ((c->policy & POLICY_DONT_REKEY)
                 && c->kind == CK_INSTANCE)
                 {
@@ -3021,10 +2814,15 @@ bool was_eroute_idle(struct state *st, time_t idle_max) {
                                 ret = TRUE; /* be paranoid */
                                 break;
                         }
-                        if(sscanf(p, "%d", (int *) &idle_time) <= 0) {
+			{
+			    int idle_time_int;
+
+			    if(sscanf(p, "%d", &idle_time_int) <= 0) {
                                 ret = TRUE;
                                 break;
-                        }
+			    }
+			    idle_time = idle_time_int;
+			}
                         if(idle_time > idle_max) {
                                DBG(DBG_KLIPS,
                                 DBG_log("SA %s found idle for more than %ld sec",
