@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: vendor.c,v 1.34 2004/11/30 02:29:07 mcr Exp $
+ * RCSID $Id: vendor.c,v 1.43.2.1 2005/07/26 02:05:10 ken Exp $
  */
 
 #include <stdlib.h>
@@ -36,6 +36,7 @@
 #include "connections.h"
 #include "packet.h"
 #include "demux.h"
+#include "server.h"
 #include "whack.h"
 #include "vendor.h"
 #include "quirks.h"
@@ -74,8 +75,11 @@
  *  6d761ddc26aceca1b0ed11fabbb860c4
  *  5946c258f99a1a57b03eb9d1759e0f24 (From a Cisco VPN 3k)
  *  ebbc5b00141d0c895e11bd395902d690 (From a Cisco VPN 3k)
+ *  3e984048101e66cc659fd002b0ed3655 (From a Cisco 1800 IOS device)
  *
  * Microsoft L2TP (???):
+ * (This could be the MSL2TP client, which is a stripped version of SafeNet)
+ *
  *  47bbe7c993f1fc13b4e6d0db565c68e5010201010201010310382e312e3020284275696c6420313029000000
  *  >> 382e312e3020284275696c6420313029 = '8.1.0 (Build 10)'
  *  3025dbd21062b9e53dc441c6aab5293600000000
@@ -85,14 +89,28 @@
  *    da8e937880010000
  *    404bf439522ca3f6
  *
+ * NCP.de
+ *    101fb0b35c5a4f4c08b919f1cb9777b0
+ *
+ * Watchguard FireBox (II ?)
+ * da8e937880010000
+ *
+ * Nortel contivity 251 (RAS F/W Version: VA251_2.0.0.0.013 | 12/3/2003  
+ *   DSL FW Version: Alcatel, Version 3.9.122)
+ * 4485152d18b6bbcd0be8a8469579ddcc
+ * 625027749d5ab97f5616c1602765cf480a3b7d0b)
+ *
+ * Zyxel Zywall 2 / Zywall 30w
+ * 625027749d5ab97f5616c1602765cf480a3b7d0b
  */
 
 #define MAX_LOG_VID_LEN            32
 
-#define VID_KEEP                   0x0000
+#define VID_KEEP                   0x0000  
 #define VID_MD5HASH                0x0001
 #define VID_STRING                 0x0002
 #define VID_FSWAN_HASH             0x0004
+#define VID_SELF                   0x0008
 
 #define VID_SUBSTRING_DUMPHEXA     0x0100
 #define VID_SUBSTRING_DUMPASCII    0x0200
@@ -174,6 +192,13 @@ static struct vid_struct _vid_tab[] = {
 	{ VID_CISCO3K, VID_KEEP | VID_SUBSTRING_MATCH, 
           NULL, "Cisco VPN 3000 Series" , "\x1f\x07\xf7\x0e\xaa\x65\x14\xd3\xb0\xfa\x96\x54\x2a\x50", 14},
 
+	{ VID_CISCO3K, VID_KEEP | VID_SUBSTRING_MATCH, 
+          NULL, "Cisco VPN 3000 Series" , "\x1f\x07\xf7\x0e\xaa\x65\x14\xd3\xb0\xfa\x96\x54\x2a\x50", 14},
+
+	{ VID_CISCO_IOS, VID_KEEP | VID_SUBSTRING_MATCH, 
+	  NULL, "Cisco IOS Device", "\x3e\x98\x40\x48", 4},
+
+
 	/**
 	 * Timestep VID seen:
 	 *   - 54494d455354455020312053475720313532302033313520322e303145303133
@@ -195,6 +220,10 @@ static struct vid_struct _vid_tab[] = {
 		"Openswan 2.2.0",
 		"Openswan 2.2.0")
 
+	/* always make sure to include ourself! */
+	{ VID_OPENSWANSELF,VID_SELF, "","Openswan (this version)", NULL,0},
+	
+
 	/* NAT-Traversal */
 
 	DEC_MD5_VID(NATT_STENBERG_01, "draft-stenberg-ipsec-nat-traversal-01")
@@ -206,7 +235,7 @@ static struct vid_struct _vid_tab[] = {
 	/* hash in draft-ietf-ipsec-nat-t-ike-02 contains '\n'... Accept both */
 	DEC_MD5_VID_D(NATT_IETF_02_N, "draft-ietf-ipsec-nat-t-ike-02\n", "draft-ietf-ipsec-nat-t-ike-02_n")
 	DEC_MD5_VID(NATT_IETF_03, "draft-ietf-ipsec-nat-t-ike-03")
-	DEC_MD5_VID(NATT_RFC, "Testing NAT-T RFC")
+	DEC_MD5_VID(NATT_RFC, "RFC 3947")
 
 	DEC_MD5_VID(NATT_DRAFT_IETF_IPSEC_NAT_T_IKE,"draft-ietf-ipsec-nat-t-ike")
 
@@ -236,6 +265,13 @@ static struct vid_struct _vid_tab[] = {
 	DEC_MD5_VID(MISC_FRAGMENTATION, "FRAGMENTATION")
 	DEC_MD5_VID(INITIAL_CONTACT, "Vid-Initial-Contact")
 
+	/*
+	 * NCP.de
+	 */
+	{ VID_NCP, VID_KEEP, "NCP client", NULL, 
+	  "\x10\x1f\xb0\xb3\x5c\x5a\x4f\x4c\x08\xb9\x19\xf1\xcb\x97\x77\xb0", 16 },
+	
+
 	/* -- */
 	{ 0, 0, NULL, NULL, NULL, 0 }
 
@@ -256,60 +292,73 @@ void init_vendorid(void)
 	int i;
 
 	for (vid = _vid_tab; vid->id; vid++) {
-		if (vid->flags & VID_STRING) {
-			/** VendorID is a string **/
-			vid->vid = strdup(vid->data);
-			vid->vid_len = strlen(vid->data);
+	    if(vid->flags & VID_SELF) {
+		char *d;
+		vid->vid = strdup(init_pluto_vendorid());
+		vid->vid_len = strlen(vid->vid);
+		d = alloc_bytes(strlen(vid->descr)+4
+				+strlen(ipsec_version_code())
+				+strlen(compile_time_interop_options)
+				, "self-vendor ID");
+		sprintf(d, "%s %s %s"
+			, vid->descr, ipsec_version_code()
+			, compile_time_interop_options);
+		vid->descr = (const char *)d;
+	    }
+	    else if (vid->flags & VID_STRING) {
+		/** VendorID is a string **/
+		vid->vid = strdup(vid->data);
+		vid->vid_len = strlen(vid->data);
+	    }
+	    else if (vid->flags & VID_MD5HASH) {
+		/** VendorID is a string to hash with MD5 **/
+		char *vidm =  malloc(MD5_DIGEST_SIZE);
+		vid->vid = vidm;
+		if (vidm) {
+		    unsigned const char *d = vid->data;
+		    osMD5Init(&ctx);
+		    osMD5Update(&ctx, d, strlen(vid->data));
+		    osMD5Final(vidm, &ctx);
+		    vid->vid_len = MD5_DIGEST_SIZE;
 		}
-		else if (vid->flags & VID_MD5HASH) {
-			/** VendorID is a string to hash with MD5 **/
-			char *vidm =  malloc(MD5_DIGEST_SIZE);
-			vid->vid = vidm;
-			if (vidm) {
-			        unsigned const char *d = vid->data;
-				osMD5Init(&ctx);
-				osMD5Update(&ctx, d, strlen(vid->data));
-				osMD5Final(vidm, &ctx);
-				vid->vid_len = MD5_DIGEST_SIZE;
-			}
-		}
-		else if (vid->flags & VID_FSWAN_HASH) {
-			/** FreeS/WAN 2.00+ specific hash **/
+	    }
+	    else if (vid->flags & VID_FSWAN_HASH) {
+		/** FreeS/WAN 2.00+ specific hash **/
 #define FSWAN_VID_SIZE 12
-			unsigned char hash[MD5_DIGEST_SIZE];
-			char *vidm =  malloc(FSWAN_VID_SIZE);
-			vid->vid = vidm;
-			if (vidm) {
-				osMD5Init(&ctx);
-				osMD5Update(&ctx, vid->data, strlen(vid->data));
-				osMD5Final(hash, &ctx);
-				vidm[0] = 'O';
-				vidm[1] = 'E';
+		unsigned char hash[MD5_DIGEST_SIZE];
+		char *vidm =  malloc(FSWAN_VID_SIZE);
+		vid->vid = vidm;
+		if (vidm) {
+		    osMD5Init(&ctx);
+		    osMD5Update(&ctx, vid->data, strlen(vid->data));
+		    osMD5Final(hash, &ctx);
+		    vidm[0] = 'O';
+		    vidm[1] = 'E';
 #if FSWAN_VID_SIZE - 2 <= MD5_DIGEST_SIZE
-				memcpy(vidm + 2, hash, FSWAN_VID_SIZE - 2);
+		    memcpy(vidm + 2, hash, FSWAN_VID_SIZE - 2);
 #else
-				memcpy(vidm + 2, hash, MD5_DIGEST_SIZE);
-				memset(vidm + 2 + MD5_DIGEST_SIZE, '\0',
-					FSWAN_VID_SIZE - 2 - MD5_DIGEST_SIZE);
+		    memcpy(vidm + 2, hash, MD5_DIGEST_SIZE);
+		    memset(vidm + 2 + MD5_DIGEST_SIZE, '\0',
+			   FSWAN_VID_SIZE - 2 - MD5_DIGEST_SIZE);
 #endif
-				for (i = 2; i < FSWAN_VID_SIZE; i++) {
-					vidm[i] &= 0x7f;
-					vidm[i] |= 0x40;
-				}
-				vid->vid_len = FSWAN_VID_SIZE;
-			}
-		}
-
-		if (vid->descr == NULL) {
-			/** Find something to display **/
-			vid->descr = vid->data;
+		    for (i = 2; i < FSWAN_VID_SIZE; i++) {
+			vidm[i] &= 0x7f;
+			vidm[i] |= 0x40;
+		    }
+		    vid->vid_len = FSWAN_VID_SIZE;
 		}
+	    }
+	    
+	    if (vid->descr == NULL) {
+		/** Find something to display **/
+		vid->descr = vid->data;
+	    }
 #if 0
-		DBG_log("vendorid_init: %d [%s]",
-			vid->id,
-			vid->descr ? vid->descr : ""
-			);
-		if (vid->vid) DBG_dump("VID:", vid->vid, vid->vid_len);
+	    DBG_log("vendorid_init: %d [%s]",
+		    vid->id,
+		    vid->descr ? vid->descr : ""
+		);
+	    if (vid->vid) DBG_dump("VID:", vid->vid, vid->vid_len);
 #endif
 	}
 	_vid_struct_init = 1;
@@ -341,51 +390,55 @@ static void handle_known_vendorid (struct msg_digest *md UNUSED
 
 	switch (vid->id) {
 #ifdef NAT_TRAVERSAL
-		/**
-		 * Use most recent supported NAT-Traversal method and ignore
-		 * the other ones (implementations will send all supported
-		 * methods but only one will be used)
-		 *
-		 * Note: most recent == higher id in vendor.h
-		 */
-		case VID_NATT_IETF_00:
-			if (!nat_traversal_support_non_ike)
-			    break;
-			vid_usefull = 1;
-			if ((nat_traversal_enabled) && (!md->quirks.nat_traversal_vid)) {
-				md->quirks.nat_traversal_vid = vid->id;
-			}
-			break;
-		case VID_NATT_IETF_02:
-		case VID_NATT_IETF_02_N:
-		case VID_NATT_IETF_03:
-		case VID_NATT_RFC:
- 		        vid_usefull = 1;
-			if(!nat_traversal_support_port_floating) {
-			  loglog(RC_LOG_SERIOUS
-				 , "received Vendor ID payload [%s] meth=%d, "
-				 "but port floating is off"
-				 , vid->descr, vid->id);
-			  return;
-			} else {
-			  if (md->quirks.nat_traversal_vid < vid->id) {
-			    loglog(RC_LOG_SERIOUS
-				   , "received Vendor ID payload [%s] method set to=%d "
-				   , vid->descr, vid->id);
-			    md->quirks.nat_traversal_vid = vid->id;
-			    return;
-			  } else {
-			    loglog(RC_LOG_SERIOUS
-				   , "received Vendor ID payload [%s] meth=%d, "
-				   "but already using method %d"
-				   , vid->descr, vid->id
-				   , md->quirks.nat_traversal_vid);
-			    return;
-			  }
-			}
-			break;
-#endif
+	    /**
+	     * Use most recent supported NAT-Traversal method and ignore
+	     * the other ones (implementations will send all supported
+	     * methods but only one will be used)
+	     *
+	     * Note: most recent == higher id in vendor.h
+	     */
 
+	    /* PAUL TRY THIS IF BELOW FAILS WITH APPLE */
+	    /*case VID_NATT_DRAFT_IETF_IPSEC_NAT_T_IKE: */
+	case VID_NATT_IETF_00:
+	    if (!nat_traversal_support_non_ike)
+		break;
+	    vid_usefull = 1;
+	    if ((nat_traversal_enabled) && (!md->quirks.nat_traversal_vid)) {
+		md->quirks.nat_traversal_vid = vid->id;
+	    }
+	    break;
+	case VID_NATT_IETF_02:
+	case VID_NATT_IETF_02_N:
+	case VID_NATT_IETF_03:
+	case VID_NATT_DRAFT_IETF_IPSEC_NAT_T_IKE:
+	case VID_NATT_RFC:
+	    vid_usefull = 1;
+	    if(!nat_traversal_support_port_floating) {
+		loglog(RC_LOG_SERIOUS
+		       , "received Vendor ID payload [%s] meth=%d, "
+		       "but port floating is off"
+		       , vid->descr, vid->id);
+		return;
+	    } else {
+		if (md->quirks.nat_traversal_vid < vid->id) {
+		    loglog(RC_LOG_SERIOUS
+			   , "received Vendor ID payload [%s] method set to=%d "
+			   , vid->descr, vid->id);
+		    md->quirks.nat_traversal_vid = vid->id;
+		    return;
+		} else {
+		    loglog(RC_LOG_SERIOUS
+			   , "received Vendor ID payload [%s] meth=%d, "
+			   "but already using method %d"
+			   , vid->descr, vid->id
+			   , md->quirks.nat_traversal_vid);
+		    return;
+		}
+	    }
+	    break;
+#endif
+	    
         case VID_MISC_DPD:
 	    /* Remote side would like to do DPD with us on this connection */
 	    md->dpd = 1;
@@ -411,6 +464,10 @@ static void handle_known_vendorid (struct msg_digest *md UNUSED
 	    break;
 #endif
 	    
+	case VID_OPENSWANSELF:
+	    vid_usefull=1;
+	    break;
+	    
 	default:
 	    break;
 	}
@@ -537,3 +594,22 @@ bool out_vendorid (u_int8_t np, pb_stream *outs, unsigned int vid)
 		pvid->vid, pvid->vid_len, "V_ID");
 }
 
+/* OpenPGP Vendor ID needed for interoperability with PGPnet
+ *
+ * Note: it is a NUL-terminated ASCII string, but NUL won't go on the wire.
+ */
+char pgp_vendorid[] = "OpenPGP10171";
+const int pgp_vendorid_len = sizeof(pgp_vendorid);
+
+char dpd_vendorid[] = {0xAF, 0xCA, 0xD7, 0x13, 0x68, 0xA1, 0xF1,
+          0xC9, 0x6B, 0x86, 0x96, 0xFC, 0x77, 0x57, 0x01, 0x00};
+const int dpd_vendorid_len = sizeof(dpd_vendorid);
+
+
+
+/*
+ * Local Variables:
+ * c-basic-offset:4
+ * c-style: pluto
+ * End:
+ */
