@@ -11,7 +11,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: vendor.h,v 1.10 2004/11/05 00:28:20 ken Exp $
+ * RCSID $Id: vendor.h,v 1.13.2.1 2005/07/26 02:05:10 ken Exp $
  */
 
 #ifndef _VENDOR_H_
@@ -54,6 +54,9 @@ enum known_vendorid {
   VID_OPENSWAN2		    =33,
   VID_MACOSX                =34,
   VID_CISCO3K               =35,
+  VID_OPENSWANSELF	    =36,
+  VID_NCP                   =37,
+  VID_CISCO_IOS             =38,
 
 
   /* 101 - 200 : NAT-Traversal */
@@ -94,5 +97,12 @@ bool out_vendorid (u_int8_t np, pb_stream *outs, enum known_vendorid vid);
 
 extern const char compile_time_interop_options[];
 
+extern char pgp_vendorid[];
+extern const int pgp_vendorid_len;
+
+extern char dpd_vendorid[];
+extern const int dpd_vendorid_len;
+
+
 #endif /* _VENDOR_H_ */
 
